@@ -19,10 +19,6 @@ static int _fastcall put_extend_blk(int block_id, int block_len, char far * bloc
 static int _fastcall store_item_name(char *);
 static void _fastcall setup_save_info(struct fractal_info far * save_info);
 
-#ifdef XFRACT
-int decode_fractal_info();
-#endif
-
 /*
                         Save-To-Disk Routines (GIF)
 
@@ -32,7 +28,7 @@ The compression logic in this file has been replaced by the classic
 UNIX compress code. We have extensively modified the sources to fit
 Fractint's needs, but have left the original credits where they
 appear. Thanks to the original authors for making available these
-classic and reliable sources. Of course, they our not responsible for 
+classic and reliable sources. Of course, they are not responsible for 
 all the changes we have made to integrate their sources into Fractint.
 
 MEMORY ALLOCATION
@@ -58,10 +54,12 @@ static BFCODE paletteBW[] =
    0, 0, 0, 63, 63, 63,
 };
 
+#ifndef XFRACT
 static BFCODE paletteCGA[] =
 {                               /* 4-color (CGA) palette  */
    0, 0, 0, 21, 63, 63, 63, 21, 63, 63, 63, 63,
 };
+#endif
 
 static BFCODE paletteEGA[] =
 {                               /* 16-color (EGA/CGA) pal */
@@ -341,7 +339,7 @@ int encoder()
 #else
    if (colors > 2)
    {
-      if (gotrealdac)
+      if (gotrealdac || fake_lut)
       {                         /* got a DAC - must be a VGA */
          if (!shftwrite((BYTE far *) dacbox, 256))
             goto oops;
@@ -421,23 +419,27 @@ int encoder()
          if (!put_extend_blk(2, resume_len, (char far *)block))
             goto oops;
       }
-      if (save_info.fractal_type == FORMULA || save_info.fractal_type == FFORMULA)
-         save_info.tot_extend_len += store_item_name(FormName);
-      if (save_info.fractal_type == LSYSTEM)
+/* save_info.fractal_type gets modified in setup_save_info() in float only
+   version, so we need to use fractype.  JCO 06JAN01 */
+/*    if (save_info.fractal_type == FORMULA || save_info.fractal_type == FFORMULA) */
+      if (fractype == FORMULA || fractype == FFORMULA)
+           save_info.tot_extend_len += store_item_name(FormName);
+/*    if (save_info.fractal_type == LSYSTEM) */
+      if (fractype == LSYSTEM)
          save_info.tot_extend_len += store_item_name(LName);
-      if (save_info.fractal_type == IFS || save_info.fractal_type == IFS3D)
-         save_info.tot_extend_len += store_item_name(IFSName);
+/*    if (save_info.fractal_type == IFS || save_info.fractal_type == IFS3D) */
+      if (fractype == IFS || fractype == IFS3D)
+           save_info.tot_extend_len += store_item_name(IFSName);
       if (display3d <= 0 && rangeslen)
       {
          /* ranges block, 004 */
          save_info.tot_extend_len += extend_blk_len(rangeslen * 2);
 #ifdef XFRACT
          fix_ranges(ranges, rangeslen, 0);
-         put_extend_blk(4, rangeslen * 2, (char far *) ranges);
-#else
+#endif
          if (!put_extend_blk(4, rangeslen * 2, (char far *) ranges))
             goto oops;
-#endif
+
       }
       /* Extended parameters block 005 */
       if (bf_math)
@@ -457,7 +459,7 @@ int encoder()
           struct evolution_info resume_e_info;
           GENEBASE gene[NUMGENES];
           MoveFromMemory((BYTE *)&gene, (U16)sizeof(gene), 1L, 0L, gene_handle);
-          if (evolve_handle == 0) {
+          if (evolve_handle == 0 || calc_status == 4) {
              esave_info.paramrangex     = paramrangex;
              esave_info.paramrangey     = paramrangey;
              esave_info.opx             = opx;
@@ -507,8 +509,9 @@ int encoder()
           decode_evolver_info(&esave_info, 0);
 #endif
           /* evolution info block, 006 */
-          put_extend_blk(6, sizeof(esave_info), (char far *) &esave_info);
           save_info.tot_extend_len += extend_blk_len(sizeof(esave_info));
+          if (!put_extend_blk(6, sizeof(esave_info), (char far *) &esave_info))
+             goto oops;
       }
 
       /* main and last block, 001 */
@@ -595,6 +598,8 @@ static int _fastcall store_item_name(char *nameptr)
       fsave_info.uses_p3 = (short) uses_p3;
       fsave_info.uses_ismand = (short) uses_ismand;
       fsave_info.ismand = (short) ismand;
+      fsave_info.uses_p4 = (short) uses_p4;
+      fsave_info.uses_p5 = (short) uses_p5;
    }
    else
    {
@@ -603,6 +608,8 @@ static int _fastcall store_item_name(char *nameptr)
       fsave_info.uses_p3 = 0;
       fsave_info.uses_ismand = 0;
       fsave_info.ismand = 0;
+      fsave_info.uses_p4 = 0;
+      fsave_info.uses_p5 = 0;
    }
    for (i = 0; i < sizeof(fsave_info.future) / sizeof(short); i++)
       fsave_info.future[i] = 0;
@@ -618,7 +625,7 @@ static void _fastcall setup_save_info(struct fractal_info far * save_info)
       maxfn = 0;
    /* set save parameters in save structure */
    far_strcpy(save_info->info_id, INFO_ID);
-   save_info->version = 13;     /* file version, independent of system */
+   save_info->version = 15;     /* file version, independent of system */
    /* increment this EVERY time the fractal_info structure changes */
 
    if (maxit <= SHRT_MAX)
@@ -756,6 +763,7 @@ static void _fastcall setup_save_info(struct fractal_info far * save_info)
    save_info->stoppass = (short) stoppass;
    save_info->quick_calc = (short) quick_calc;
    save_info->closeprox = closeprox;
+   save_info->nobof = (short) nobof;
    for (i = 0; i < sizeof(save_info->future) / sizeof(short); i++)
       save_info->future[i] = 0;
       
