@@ -1,7 +1,8 @@
 #ifndef EXTERNS_H
 #define EXTERNS_H
 
-#ifndef DEBUG
+#if 0
+/* #ifndef DEBUG */
 #define DEBUG 1
 #endif
 
@@ -37,6 +38,7 @@ extern int                   basin;
 extern int                   bf_save_len;
 extern int                   bfdigits;
 extern int                   biomorph;
+extern unsigned int          bits;
 extern int                   bitshift;
 extern int                   bitshiftless1;
 extern BYTE                  block[];
@@ -60,6 +62,7 @@ extern int (*                calctype)(void);
 extern int                   calc_status;
 extern char                  calibrate;
 extern int                   checkcurdir;
+extern int                   chkd_vvs;
 extern long                  cimag;
 extern double                closenuff;
 extern double                closeprox;
@@ -108,9 +111,12 @@ extern long                  dely;
 extern LDBL                  delyy2;
 extern LDBL                  delyy;
 extern float                 depthfp;
+extern unsigned long         dif_counter;
+extern unsigned long         dif_limit;
 extern int                   disk16bit;
 extern int                   diskflag;
 extern int                   diskisactive;
+extern int                   disktarga;
 extern int                   diskvideo;
 extern int                   display3d;
 extern long                  distest;
@@ -256,6 +262,7 @@ extern char                  LName[];
 extern _LCMPLX               lnew;
 extern int                   loaded3d;
 extern int                   LodPtr;
+extern int                   Log_Auto_Calc;
 extern int                   Log_Calc;
 extern int                   Log_Fly_Calc;
 extern long                  LogFlag;
@@ -327,8 +334,10 @@ extern char                  newodpy;
 extern double                newopx;
 extern double                newopy;
 extern int                   neworbittype;
+extern int                   nextsavedincr;
 extern int                   no_sub_images;
 extern int                   no_mag_calc;
+extern int                   nobof;
 extern int                   numaffine;
 extern unsigned              numcolors;
 extern const int             numtrigfn;
@@ -634,6 +643,7 @@ extern char                  s_miim[];
 extern char                  s_mod[];
 extern char                  s_mono[];
 extern char                  s_mult[];
+extern char                  s_nobof[];
 extern char                  s_none[];
 extern char                  s_noninterlaced[];
 extern char                  s_normal[];
@@ -709,6 +719,7 @@ extern char                  s_tanh[];
 extern char                  s_tan[];
 extern char                  s_targa_out[];
 extern char                  s_targa_overlay[];
+extern char                  s_tdis[];
 extern char                  s_tempdir[];
 extern char                  s_textcolors[];
 extern char                  s_textsafe[];
@@ -773,6 +784,7 @@ extern int                   TPlusErr;
 extern int                   transparent[];
 extern BYTE                  trigndx[];
 extern int                   truecolor;
+extern int                   truemode;
 extern char                  tstack[];
 extern double                twopi;
 extern VOIDFARPTR            typespecific_workarea;
@@ -784,6 +796,8 @@ extern short                 uses_ismand;
 extern short                 uses_p1;
 extern short                 uses_p2;
 extern short                 uses_p3;
+extern short                 uses_p4;
+extern short                 uses_p5;
 extern int                   use_old_distest;
 extern int                   use_old_period;
 extern int                   using_jiim;
@@ -793,9 +807,16 @@ extern char                  usr_floatflag;
 extern int                   usr_periodicitycheck;
 extern char                  usr_stdcalcmode;
 extern int                   vesa_detect;
+extern int                   vesa_xres;
+extern int                   vesa_yres;
 extern struct videoinfo      videoentry;
 extern VIDEOINFO             videotable[];
+extern int                   video_cutboth;
+extern int                   video_scroll;
+extern int                   video_startx;
+extern int                   video_starty;
 extern int                   video_type;
+extern int                   video_vram;
 extern VIDEOINFO *           vidtbl;
 extern int                   vidtbllen;
 extern VECTOR                view;
@@ -804,6 +825,7 @@ extern float                 viewreduction;
 extern int                   viewwindow;
 extern int                   viewxdots;
 extern int                   viewydots;
+extern int                   virtual;
 extern unsigned              vsp;
 extern int                   vxdots;
 extern int                   whichimage;
@@ -853,9 +875,12 @@ extern double                zdepth;
 extern int                   zdots;
 extern int                   zoomoff;
 extern int                   zrotate;
+extern int                   zscroll;
 extern double                zskew;
 extern double                zwidth;
-extern unsigned long         dif_counter;
-extern unsigned long         dif_limit;
-extern unsigned              bits;
+
+#ifdef XFRACT
+extern  int                  fake_lut;
+#endif
+
 #endif
