@@ -21,6 +21,14 @@
  *----------------------------------------------------------------------
  */
 
+double _2_ = 2.0;
+double _1_ = 1.0;
+double PointFive = 0.5;
+/* infinity is declared in sunmath.h */
+#ifndef USE_SUNMATH
+double infinity = 1.0E+300;
+#endif
+
 void FPUaptan387(double *y, double *x, double *atan)
 {
     *atan = atan2(*y,*x);
@@ -36,7 +44,7 @@ void FPUcplxmul(_CMPLX *x, _CMPLX *y, _CMPLX *z)
 
 void FPUcplxdiv(_CMPLX *x, _CMPLX *y, _CMPLX *z)
 {
-    double mod,tx,ty,yxmod,yymod;
+    double mod,tx,yxmod,yymod;
     mod = y->x * y->x + y->y * y->y;
     if (mod==0) {
 	DivideOverflow++;
