@@ -31,11 +31,7 @@ extern int tabmode;
 
 int DivideOverflow = 0;
 int cpu=0;		/* cpu type: 86, 186, 286, or 386 */
-#ifndef XFRACT
 int fpu=0;		/* fpu type: 0, 87, 287, 387 */
-#else
-int fpu=1;
-#endif
 
 SEGTYPE extraseg=0;		/* extra 64K segment (allocated by init) */
 /* ********************** Mouse Support Variables ************************** */
@@ -80,7 +76,7 @@ int cmpcount;
 ; ****************** Function initasmvars() *****************************
 */
 void
-initasmvars()
+initasmvars(void)
 {
     if (cpu!=0) return;
     overflow = 0;
@@ -90,12 +86,7 @@ initasmvars()
     cpu = 1;
 
     /* set fpu type */
-#ifndef XFRACT
-    fpu = 0;
-#else
-    fpu = 1;
-#endif
-
+   /* not needed, set fpu in sstools.ini */
 }
 
 void fpe_handler(int signum)
@@ -114,13 +105,7 @@ void fpe_handler(int signum)
 ;       z = multiply(x,y,n)
 ;
 */
-static float
-tofloat(x,n)
-long x;
-int n;
-{
-    return (float)x/(float)(1<<n);
-}
+
 /*
  * 32 bit integer multiply with n bit shift.
  * Note that we fake integer multiplication with floating point
@@ -175,7 +160,12 @@ int n;
 ;       Think of 'keypressed()' as a super-'kbhit()'.
 */
 int keybuffer = 0;
-keypressed() {
+
+int getkeynowait(void);
+int getkeyint(int);
+
+int
+keypressed(void) {
     int ch;
     ch = getkeynowait();
     if (!ch) return 0;
@@ -199,10 +189,10 @@ keypressed() {
  * while (!keypressed()) {}
  * If timeout=1, waitkeypressed will time out after .5 sec.
  */
+int
 waitkeypressed(timeout)
 int timeout;
 {
-    int status;
     while (!keybuffer) {
 	keybuffer = getkeyint(1);
 	if (timeout) break;
@@ -213,7 +203,8 @@ int timeout;
 /*
  * This routine returns a key, ignoring F1
  */
-getakeynohelp() {
+int
+getakeynohelp(void) {
     int ch;
     while (1) {
 	ch = getakey();
@@ -221,10 +212,11 @@ getakeynohelp() {
     }
     return ch;
 }
-/* 
+/*
  * This routine returns a keypress
  */
-getakey()
+int
+getakey(void)
 {
     int ch;
 
@@ -237,7 +229,8 @@ getakey()
 /*
  * This routine returns the current key, or 0.
  */
-getkeynowait() {
+int
+getkeynowait(void) {
     return getkeyint(0);
 }
 
@@ -248,6 +241,7 @@ getkeynowait() {
  * We also have to handle the slide file, etc.
  */
 
+int
 getkeyint(block)
 int block;
 {
@@ -301,10 +295,12 @@ void
 buzzer(buzzertype)
 int buzzertype;
 {
-    printf("\007");
-    fflush(stdout);
+    if ((soundflag & 7) != 0) {
+        printf("\007");
+        fflush(stdout);
+    }
     if (buzzertype==0) {
-	redrawscreen();
+        redrawscreen();
     }
 }
 
@@ -352,14 +348,14 @@ int hertz;
 }
 
 void
-nosnd()
+nosnd(void)
 {}
 
 /*
 ; long readticker() returns current bios ticker value
 */
 long
-readticker()
+readticker(void)
 {
     return clock_ticks();
 }
@@ -444,7 +440,7 @@ char *a, *b;
     return strcmp(a,b);
 }
 
-int 
+int
 far_stricmp(a,b)
 char *a,*b;
 {
@@ -499,60 +495,6 @@ int len;
     memicmp(a,b,len);
 }
 
-/*
- * Extended memory support
- */
-
-BYTE *
-emmquery()
-{
-}
-
-unsigned int
-emmgetfree()
-{}
-
-unsigned int
-emmallocate(len)
-unsigned int len;
-{}
-
-void
-emmdeallocate(len)
-unsigned int len;
-{}
-
-void
-emmgetpage(a,b)
-unsigned int a, b;
-{}
-
-void
-emmclearpage(a,b)
-unsigned int a, b;
-{}
-
-
-unsigned int *
-xmmquery()
-{}
-
-unsigned int
-xmmallocate(a)
-unsigned int a;
-{}
-
-void
-xmmdeallocate(a)
-unsigned int a;
-{}
-
-unsigned int
-xmmmoveextended(a)
-struct XMM_Move *a;
-{}
-
-
 /* --------------------------------------------------------------------
  * The following routines are used for encoding/decoding gif images.
  * If we aren't on a PC, things are rough for decoding the fractal info
@@ -561,8 +503,9 @@ struct XMM_Move *a;
  * If dir==0, we convert to MSDOS form.  Otherwise we convert from MSDOS.
  */
 
-static getChar(), getInt(), getLong(), getFloat(), getDouble();
+static void getChar(), getInt(), getLong(), getFloat(), getDouble();
 
+void
 decode_fractal_info(info,dir)
     struct fractal_info *info;
     int dir;
@@ -706,6 +649,7 @@ decode_fractal_info(info,dir)
     getInt(&info->stoppass,&bufPtr,dir);
     getInt(&info->quick_calc,&bufPtr,dir);
     getDouble(&info->closeprox,&bufPtr,dir);
+    getInt(&info->nobof,&bufPtr,dir);
 
     for (i=0;i<(sizeof(info->future)/sizeof(short));i++) {
         getInt(&info->future[i],&bufPtr,dir);
@@ -726,7 +670,7 @@ decode_fractal_info(info,dir)
  * This routine gets a char out of the buffer.
  * It updates the buffer pointer accordingly.
  */
-static getChar(dst,src,dir)
+static void getChar(dst,src,dir)
     unsigned char *dst;
     unsigned char **src;
     int dir;
@@ -743,7 +687,7 @@ static getChar(dst,src,dir)
  * This routine gets an int out of the buffer.
  * It updates the buffer pointer accordingly.
  */
-static getInt(dst,src,dir)
+static void getInt(dst,src,dir)
     short *dst;
     unsigned char **src;
     int dir;
@@ -761,7 +705,7 @@ static getInt(dst,src,dir)
  * This routine gets a long out of the buffer.
  * It updates the buffer pointer accordingly.
  */
-static getLong(dst,src,dir)
+static void getLong(dst,src,dir)
     long *dst;
     unsigned char **src;
     int dir;
@@ -802,7 +746,7 @@ static getLong(dst,src,dir)
  * buffer;
  * It updates the buffer pointer accordingly.
  */
-static getDouble(dst,src,dir)
+static void getDouble(dst,src,dir)
     double *dst;
     unsigned char **src;
     int dir;
@@ -877,7 +821,7 @@ static getDouble(dst,src,dir)
  * This routine gets a float out of the buffer.
  * It updates the buffer pointer accordingly.
  */
-static getFloat(dst,src,dir)
+static void getFloat(dst,src,dir)
     float *dst;
     unsigned char **src;
     int dir;
@@ -893,10 +837,10 @@ static getFloat(dst,src,dir)
 	    *dst = 0;
 	} else {
 #ifdef __SVR4
-	    e = (((*src)[3]&0x7f)<<1)| ((int)((*src)[2]&0x80)>>7) - 127;
+	    e = ((((*src)[3]&0x7f)<<1) | ((int)((*src)[2]&0x80)>>7)) - 127;
 	    f = 1 + (int)((*src)[2]&0x7f)/P7 + (int)((*src)[1])/P15 + (int)((*src)[0])/P23;
 #else
-      e = (((*src)[3]&0x7f)<<1)| (((*src)[2]&0x80)>>7) - 127;
+      e = ((((*src)[3]&0x7f)<<1) | (((*src)[2]&0x80)>>7)) - 127;
       f = 1 + ((*src)[2]&0x7f)/P7 + (*src)[1]/P15 + (*src)[0]/P23;
 #endif
 	    f *= pow(2.,(double)e);
@@ -940,6 +884,7 @@ static getFloat(dst,src,dir)
 /*
  * Fix up the ranges data.
  */
+void
 fix_ranges(ranges,num,dir)
     int *ranges, num;
     int dir;
@@ -958,11 +903,12 @@ fix_ranges(ranges,num,dir)
 	bcopy((char *)ranges, (char *)buf, num*sizeof(int));
     }
     for (i=0;i<num;i++) {
-	getInt(&ranges[i],bufPtr,dir);
+	getInt(&ranges[i],&bufPtr,dir);
     }
     free((char *)buf);
 }
 
+void
 decode_evolver_info(info,dir)
     struct evolution_info *info;
     int dir;
