@@ -5,9 +5,6 @@
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#ifdef XFRACT
-#include <unistd.h>
-#endif
   /* see Fractint.c for a description of the "include"  hierarchy */
 #include "port.h"
 #include "prototyp.h"
@@ -38,10 +35,6 @@ short skipxdots,skipydots;      /* for decoder, when reducing image */
 int bad_outside = 0;
 int ldcheck = 0;
 
-#ifdef XFRACT
-int decode_fractal_info();
-#endif
-
 int read_overlay()      /* read overlay/3D files, if reqr'd */
 {
    struct fractal_info read_info;
@@ -309,6 +302,15 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
             functionpreloaded = 1;
    }
 
+   nobof=0;
+   if (read_info.version > 13) { /* post-version 20.1.2 */
+      nobof = read_info.nobof;
+   }
+
+   /* if (read_info.version > 14)  post-version 20.1.12 */
+   /* modified saved evolver structure JCO 12JUL01 */
+   Log_Auto_Calc = 0;  /* make sure it's turned off */
+
    backwards_v18();
    backwards_v19();
    backwards_v20();
@@ -336,6 +338,10 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
       display3d = olddisplay3d;
       floatflag = oldfloatflag;
       if (i) {
+         if(blk_2_info.got_data == 1) {
+            MemoryRelease((U16)blk_2_info.resume_data);
+            blk_2_info.length = 0;
+         }
          initmode = -1;
          return(-1);
          }
@@ -382,6 +388,8 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
                 uses_p3 = blk_3_info.uses_p3;
                 uses_ismand = blk_3_info.uses_ismand;
                 ismand = blk_3_info.ismand;
+                uses_p4 = blk_3_info.uses_p4;
+                uses_p5 = blk_3_info.uses_p5;
                 break;
              }
           blk_3_info.form_name[ITEMNAMELEN] = 0;
@@ -399,7 +407,7 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
           {
           ranges = (int far *)blk_4_info.range_data;
           rangeslen = blk_4_info.length;
-#ifdef XFRACTINT
+#ifdef XFRACT
           fix_ranges(ranges,rangeslen,1);
 #endif
           }
@@ -423,7 +431,12 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
           if (gene_handle == 0)
              gene_handle = MemoryAlloc((U16)sizeof(gene),1L,FARMEM);
           MoveFromMemory((BYTE *)&gene, (U16)sizeof(gene), 1L, 0L, gene_handle);
-          if (blk_6_info.ecount != blk_6_info.gridsz * blk_6_info.gridsz) {
+          if (read_info.version < 15)  /* This is VERY Ugly!  JCO  14JUL01 */
+             /* Increasing NUMGENES moves ecount in the data structure */
+             /* We added 4 to NUMGENES, so ecount is at NUMGENES-4 */
+             blk_6_info.ecount = blk_6_info.mutate[NUMGENES-4];
+          if (blk_6_info.ecount != blk_6_info.gridsz * blk_6_info.gridsz
+             && calc_status != 4) {
              calc_status = 2;
              if (evolve_handle == 0)
                 evolve_handle = MemoryAlloc((U16)sizeof(resume_e_info),1L,FARMEM);
@@ -445,6 +458,11 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
              resume_e_info.fiddlefactor = blk_6_info.fiddlefactor;
              resume_e_info.ecount       = blk_6_info.ecount;
              MoveToMemory((BYTE *)&resume_e_info,(U16)sizeof(resume_e_info),1L,0L,evolve_handle);
+          } else {
+             if (evolve_handle != 0)  /* Image completed, release it. */
+                MemoryRelease(evolve_handle);
+             evolve_handle = 0;
+             calc_status = 4;
           }
           paramrangex  = blk_6_info.paramrangex;
           paramrangey  = blk_6_info.paramrangey;
@@ -464,8 +482,17 @@ int read_overlay()      /* read overlay/3D files, if reqr'd */
           evolving = viewwindow = (int)blk_6_info.evolving;
           dpx=paramrangex/(gridsz-1);
           dpy=paramrangey/(gridsz-1);
-          for (i = 0; i < NUMGENES; i++)
-             gene[i].mutate = (int)blk_6_info.mutate[i];
+          if (read_info.version > 14)
+             for (i = 0; i < NUMGENES; i++)
+                gene[i].mutate = (int)blk_6_info.mutate[i];
+          else {
+             for (i = 0; i < 6; i++)
+                gene[i].mutate = (int)blk_6_info.mutate[i];
+             for (i = 6; i < 10; i++)
+                gene[i].mutate = 0;
+             for (i = 10; i < NUMGENES; i++)
+                gene[i].mutate = (int)blk_6_info.mutate[i-4];
+          }
           MoveToMemory((BYTE *)&gene, (U16)sizeof(gene), 1L, 0L, gene_handle);
           param_history(0); /* store history */
    }
@@ -490,7 +517,7 @@ static int find_fractal_info(char *gif_file,struct fractal_info *info,
    int hdr_offset;
    struct formula_info fload_info;
    struct evolution_info eload_info;
-   int i, j, k;
+   int i, j, k = 0;
 
    blk_2_info->got_data = 0; /* initialize to no data */
    blk_3_info->got_data = 0; /* initialize to no data */
@@ -651,6 +678,8 @@ static int find_fractal_info(char *gif_file,struct fractal_info *info,
                      blk_3_info->uses_p3 = 1;
                      blk_3_info->uses_ismand = 0;
                      blk_3_info->ismand = 1;
+                     blk_3_info->uses_p4 = 0;
+                     blk_3_info->uses_p5 = 0;
                   }
                   else {
                      blk_3_info->uses_p1 = fload_info.uses_p1;
@@ -658,6 +687,8 @@ static int find_fractal_info(char *gif_file,struct fractal_info *info,
                      blk_3_info->uses_p3 = fload_info.uses_p3;
                      blk_3_info->uses_ismand = fload_info.uses_ismand;
                      blk_3_info->ismand = fload_info.ismand;
+                     blk_3_info->uses_p4 = fload_info.uses_p4;
+                     blk_3_info->uses_p5 = fload_info.uses_p5;
                   }
                   break;
                case 4: /* ranges info */
@@ -934,16 +965,18 @@ void backwards_v18(void)
 
 void backwards_v19(void)
 {
-  if(fractype==MARKSJULIA && save_release < 1825)
+  if(fractype==MARKSJULIA && save_release < 1825) {
     if(param[2] == 0)
        param[2] = 2;
     else
        param[2] += 1;
-  if(fractype==MARKSJULIAFP && save_release < 1825)
+  }
+  if(fractype==MARKSJULIAFP && save_release < 1825) {
     if(param[2] == 0)
        param[2] = 2;
     else
        param[2] += 1;
+  }
   if((fractype==FORMULA || fractype==FFORMULA) && save_release < 1824)
     inversion[0] = inversion[1] = inversion[2] = invert = 0;
   if(fix_bof())
@@ -990,7 +1023,7 @@ int ret = 0;
        fix_bof() || fix_period_bof() || use_old_distest || decomp[0] == 2 ||
        (fractype == FORMULA && save_release <= 1920) ||
        (fractype == FFORMULA && save_release <= 1920) ||
-       (LogFlag != 0 && save_release <= 1920) ||
+       (LogFlag != 0 && save_release <= 2001) ||
        (fractype == TRIGSQR && save_release < 1900) ||
        (inside == STARTRAIL && save_release < 1825) ||
        (maxit > 32767 && save_release <= 1950) ||
@@ -1000,7 +1033,11 @@ int ret = 0;
        (fractype == FPPOPCORN && save_release <= 1960) ||
        (fractype == LPOPCORN && save_release <= 1960) ||
        (fractype == FPPOPCORNJUL && save_release <= 1960) ||
-       (fractype == LPOPCORNJUL && save_release <= 1960) )
+       (fractype == LPOPCORNJUL && save_release <= 1960) ||
+       (inside == FMODI && save_release <= 2000) ||
+       ((inside == ATANI || outside == ATAN) && save_release <= 2002) ||
+       (fractype == LAMBDATRIGFP && trigndx[0] == EXP && save_release <= 2002)
+       )
      ret = 1;
    return(ret);
 }
@@ -1356,7 +1393,7 @@ rescan:  /* entry for changed browse parms */
          i = field_prompt(0,mesg,NULL,newname,60,NULL);
          unstackscreen();
          if( i != -1)
-          if (!rename(tmpmask,newname))
+          if (!rename(tmpmask,newname)) {
             if (errno == EACCES)
             {
                static FCODE msg[] = {"sorry....can't rename"};
@@ -1369,6 +1406,7 @@ rescan:  /* entry for changed browse parms */
            check_history(oldname,tmpmask);
            far_strcpy(winlist.name,tmpmask);
            }
+          }
          MoveToMemory(winlistptr,(U16)sizeof(struct window),1L,(long)index,browsehandle);
          showtempmsg(winlist.name);
          break;
@@ -1442,8 +1480,10 @@ rescan:  /* entry for changed browse parms */
 
 static void drawindow(int colour,struct window *info)
 {
+#ifndef XFRACT
    int cross_size;
    struct coords ibl,itr;
+#endif
 
    boxcolor=colour;
    boxcount = 0;
@@ -1693,6 +1733,9 @@ static char is_visible_window
 static char paramsOK( struct fractal_info *info )
 {
 double tmpparm3, tmpparm4;
+double tmpparm5, tmpparm6;
+double tmpparm7, tmpparm8;
+double tmpparm9, tmpparm10;
 #define MINDIF 0.001
 
    if( info->version > 6) {
@@ -1705,10 +1748,32 @@ double tmpparm3, tmpparm4;
      tmpparm4 = info->parm4;
      roundfloatd(&tmpparm4);
    }
+   if( info->version > 8) {
+     tmpparm5 = info->dparm5;
+     tmpparm6 = info->dparm6;
+     tmpparm7 = info->dparm7;
+     tmpparm8 = info->dparm8;
+     tmpparm9 = info->dparm9;
+     tmpparm10 = info->dparm10;
+   }
+   else{
+     tmpparm5 = 0.0;
+     tmpparm6 = 0.0;
+     tmpparm7 = 0.0;
+     tmpparm8 = 0.0;
+     tmpparm9 = 0.0;
+     tmpparm10 = 0.0;
+   }
    if( fabs(info->creal - param[0]) < MINDIF &&
        fabs(info->cimag - param[1]) < MINDIF &&
        fabs(tmpparm3 - param[2]) < MINDIF &&
        fabs(tmpparm4 - param[3]) < MINDIF &&
+       fabs(tmpparm5 - param[4]) < MINDIF &&
+       fabs(tmpparm6 - param[5]) < MINDIF &&
+       fabs(tmpparm7 - param[6]) < MINDIF &&
+       fabs(tmpparm8 - param[7]) < MINDIF &&
+       fabs(tmpparm9 - param[8]) < MINDIF &&
+       fabs(tmpparm10 - param[9]) < MINDIF &&
        info->invert[0] - inversion[0] < MINDIF)
       return(1); /* parameters are in range */
    else
