@@ -5,14 +5,19 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
+
 #ifndef XFRACT
 #include <malloc.h>
 #include <process.h>
-#include <stdarg.h>
 #include <io.h>
+#endif
+
+#ifndef USE_VARARGS
+#include <stdarg.h>
 #else
 #include <varargs.h>
 #endif
+
   /* see Fractint.c for a description of the "include"  hierarchy */
 #include "port.h"
 #include "prototyp.h"
@@ -24,7 +29,7 @@
 static void write_batch_parms(char *colorinf,int colorsonly, int maxcolor,int i, int j);
 static void expand_comments(char far *target, char far *source);
 
-#ifndef XFRACT
+#ifndef USE_VARARGS
 static void put_parm(char *parm,...);
 #else
 static void put_parm();
@@ -36,7 +41,9 @@ int getprecbf(int);
 static void put_float(int,double,int);
 static void put_bf(int slash,bf_t r, int prec);
 static void put_filename(char *keyword,char *fname);
+#ifndef XFRACT
 static int check_modekey(int curkey,int choice);
+#endif
 static int entcompare(VOIDCONSTPTR p1,VOIDCONSTPTR p2);
 static void update_fractint_cfg(void);
 static void strip_zeros(char *buf);
@@ -84,19 +91,19 @@ void make_batch_file()
    int colorsonly = 0;
    static char far hdg[]={"Save Current Parameters"};
    /** added for pieces feature **/
-   double pdelx;
-   double pdely;
-   double pdelx2;
-   double pdely2;
+   double pdelx = 0.0;
+   double pdely = 0.0;
+   double pdelx2 = 0.0;
+   double pdely2 = 0.0;
    unsigned int pxdots, pydots, xm, ym;
-   double pxxmin, pyymax;
+   double pxxmin = 0.0, pyymax = 0.0;
    char vidmde[5];
    int promptnum;
    int piecespromts;
-   int have3rd;
+   int have3rd = 0;
    /****/
 
-   int i, j;
+   int i,j;
    char far *inpcommandfile, far *inpcommandname;
    char far *inpcomment[4];
    struct fullscreenvalues paramvalues[18];
@@ -104,17 +111,17 @@ void make_batch_file()
    char far *ptr;
    int gotinfile;
    char outname[FILE_MAX_PATH+1], buf[256], buf2[128];
-   FILE *infile;
+   FILE *infile = NULL;
    FILE *fpbat = NULL;
    char colorspec[14];
    int maxcolor;
-   int maxcolorindex;
-   char *sptr, *sptr2;
+   int maxcolorindex = 0;
+   char *sptr = NULL, *sptr2;
    int oldhelpmode;
 
    if(s_makepar[1] == 0) /* makepar map case */
       colorsonly = 1;
-      
+
    /* put comment storage in extraseg */
    inpcommandfile = MK_FP(extraseg,0);
    inpcommandname = inpcommandfile+80;
@@ -131,7 +138,7 @@ void make_batch_file()
 
    maxcolor = colors;
    strcpy(colorspec,"y");
-   if (gotrealdac && !reallyega)
+   if ((gotrealdac && !reallyega) || (istruecolor && !truemode) || fake_lut)
    {
       --maxcolor;
 /*    if (maxit < maxcolor)  remove 2 lines */
@@ -214,7 +221,7 @@ prompt_user:
       LOADBATCHPROMPTS("Fourth comment");
       paramvalues[promptnum].type = 0x100 + MAXCMT - 1;
       paramvalues[promptnum++].uval.sbuf = inpcomment[3];
-      if (gotrealdac && !reallyega)
+      if ((gotrealdac && !reallyega) || (istruecolor && !truemode) || fake_lut)
       {
          LOADBATCHPROMPTS("Record colors?");
          paramvalues[promptnum].type = 0x100 + 13;
@@ -263,7 +270,7 @@ prompt_user:
       far_strcpy(CommandName, inpcommandname);
       for(i=0;i<4;i++)
          far_strncpy(CommandComment[i], inpcomment[i], MAXCMT);
-      if (gotrealdac && !reallyega)
+      if ((gotrealdac && !reallyega) || (istruecolor && !truemode) || fake_lut)
          if (paramvalues[maxcolorindex].uval.ival > 0 &&
              paramvalues[maxcolorindex].uval.ival <= 256)
             maxcolor = paramvalues[maxcolorindex].uval.ival;
@@ -282,11 +289,11 @@ prompt_user:
 
       /* sanity checks */
       {
-      int i;
       long xtotal, ytotal;
+#ifndef XFRACT
+      int i;
 
       /* get resolution from the video name (which must be valid) */
-#ifndef XFRACT
       pxdots = pydots = 0;
       if ((i = check_vidmode_keyname(vidmde)) > 0)
           if ((i = check_vidmode_key(0, i)) >= 0) {
@@ -478,8 +485,8 @@ Continue to replace it, Cancel to back out"};
                if (CommandComment[k][0])
                   fprintf(parmfile, "%s%s\n", buf, CommandComment[k]);
             if (patchlevel != 0 && colorsonly == 0)
-               fprintf(parmfile, "%s Version %d Patchlevel %d\n", buf,
-                  release, patchlevel); 
+               fprintf(parmfile, "%s %s Version %d Patchlevel %d\n", buf,
+                  Fractint, release, patchlevel); 
          }
          write_batch_parms(colorspec, colorsonly, maxcolor, i, j);
          if(xm > 1 || ym > 1)
@@ -641,7 +648,8 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
          {
             cvtcentermag(&Xctr, &Yctr, &Magnification, &Xmagfactor, &Rotation, &Skew);
             put_parm(" %s=",s_centermag);
-            put_parm((delmin > 1000) ? "%g/%g" : "%+20.17lf/%+20.17lf", Xctr, Yctr);
+/*          convert 1000 fudged long to double, 1000/1<<24 = 6e-5 */
+            put_parm(ddelmin > 6e-5 ? "%g/%g" : "%+20.17lf/%+20.17lf", Xctr, Yctr);
          }
 #ifdef USE_LONG_DOUBLE
          put_parm("/%.7Lg",Magnification); /* precision of magnification not critical, but magnitude is */
@@ -653,8 +661,13 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
          if (fabs(Xmagfactor) > 0.5) /* or so, exact value isn't important */
             Xmagfactor = (sign(Xmagfactor) * (long)(fabs(Xmagfactor) * 1e4 + 0.5)) / 1e4;
          /* Just truncate these angles.  Who cares about 1/1000 of a degree */
+         /* Somebody does.  Some rotated and/or skewed images are slightly */
+         /* off when recreated from a PAR using 1/1000. */
+         /* JCO 08052001 */
+#if 0
          Rotation   = (long)(Rotation   * 1e3)/1e3;
          Skew       = (long)(Skew       * 1e3)/1e3;
+#endif
          if (Xmagfactor != 1 || Rotation != 0 || Skew != 0)
          { /* Only put what is necessary */
             /* The difference with Xmagfactor is that it is normally */
@@ -668,10 +681,12 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
                /* Use precision=6 here.  These angle have already been rounded        */
                /* to 3 decimal places, but angles like 123.456 degrees need 6         */
                /* sig figs to get 3 decimal places.  Trailing 0's are dropped anyway. */
-               put_float(1,Rotation,6);
+               /* Changed to 18 to address rotated and skewed problem w/ PARs */
+               /* JCO 08052001 */
+               put_float(1,Rotation,18);
                if (Skew != 0)
                {
-                  put_float(1,Skew,6);
+                  put_float(1,Skew,18);
                }
             }
          }
@@ -779,46 +794,51 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
          put_parm(" %s=",s_inside);
          if (inside == -1)
             put_parm( s_maxiter);
-         else if (inside == -59)
+         else if (inside == ZMAG)
             put_parm(s_zmag);
-         else if (inside == -60)
+         else if (inside == BOF60)
             put_parm(s_bof60);
-         else if (inside == -61)
+         else if (inside == BOF61)
             put_parm(s_bof61);
-         else if (inside == -100)
+         else if (inside == EPSCROSS)
             put_parm(s_epscross);
-         else if (inside == -101)
+         else if (inside == STARTRAIL)
             put_parm(s_startrail);
-         else if (inside == -102)
+         else if (inside == PERIOD)
             put_parm(s_period);
-         else if (inside == -103)
+         else if (inside == FMODI)
             put_parm(s_fmod);
-         else if (inside == -104)
+         else if (inside == ATANI)
             put_parm(s_atan);
          else
             put_parm( "%d",inside);
          }
-      if (closeprox != 0.01 && (inside == EPSCROSS || inside == FMOD) ) {
+      if (closeprox != 0.01 && (inside == EPSCROSS || inside == FMODI
+          || outside==FMOD) ) {
          put_parm(" %s=%.15g",s_prox,closeprox);
          }
       if (outside != -1)
       {
          put_parm(" %s=",s_outside);
-         if (outside == -2)
+         if (outside == REAL)
             put_parm(s_real);
-         else if (outside == -3)
+         else if (outside == IMAG)
             put_parm(s_imag);
-         else if (outside == -4)
+         else if (outside == MULT)
             put_parm(s_mult);
-         else if (outside == -5)
+         else if (outside == SUM)
             put_parm(s_sum);
-         else if (outside == -6)
+         else if (outside == ATAN)
             put_parm(s_atan);
+         else if (outside == FMOD)
+            put_parm(s_fmod);
+         else if (outside == TDIS)
+            put_parm(s_tdis);
          else
             put_parm( "%d",outside);
           }
 
-      if(LogFlag) {
+      if(LogFlag && !rangeslen) {
          put_parm( " %s=",s_logmap);
          if(LogFlag == -1)
             put_parm( "old");
@@ -828,7 +848,7 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
             put_parm( "%ld", LogFlag);
          }
 
-      if(Log_Fly_Calc && LogFlag) {
+      if(Log_Fly_Calc && LogFlag && !rangeslen) {
          put_parm( " %s=",s_logmode);
          if(Log_Fly_Calc == 1)
             put_parm( "fly");
@@ -946,6 +966,9 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
          put_parm( s_seqd,s_ambient,Ambient);
       if (haze)
          put_parm( s_seqd,s_haze,haze);
+      if (back_color[0] != 51 || back_color[1] != 153 || back_color[2] != 200)
+         put_parm( s_seqddd,s_background,back_color[0],back_color[1],
+                   back_color[2]);
       }
 
    if (display3d) {             /* universal 3d */
@@ -1031,7 +1054,7 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
      put_parm(s_seqd,s_polyphony,polyphony+1);
    
    if(fm_wavetype !=0)
-     put_parm(s_seqd,s_wavetype,fm_wavetype); 
+     put_parm(s_seqd,s_wavetype,fm_wavetype);
    
    if(fm_attack != 5)
       put_parm(s_seqd,s_attack,fm_attack);
@@ -1055,6 +1078,9 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
 
 #endif
 
+   if(nobof > 0)
+      put_parm(s_seqs,s_nobof,s_yes);
+
    if(orbit_delay > 0)
       put_parm(s_seqd,s_orbitdelay,orbit_delay);
 
@@ -1063,7 +1089,7 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
 
    }
 
-   if (*colorinf != 'n') 
+   if (*colorinf != 'n')
    {
       if(recordcolors=='c' && *colorinf == '@')
       {
@@ -1071,7 +1097,7 @@ static void write_batch_parms(char *colorinf, int colorsonly, int maxcolor, int
          put_parm("; %s=",s_colors);
          put_parm(colorinf);
          put_parm_line();
-      }   
+      }
 docolors:
       put_parm(" %s=",s_colors);
       if (recordcolors !='c' && recordcolors != 'y' && *colorinf == '@')
@@ -1080,6 +1106,9 @@ docolors:
          int curc,scanc,force,diffmag = -1;
          int delta,diff1[4][3],diff2[4][3];
          curc = force = 0;
+#ifdef XFRACT
+         if (fake_lut && !truemode) loaddac(); /* stupid kludge JCO 6/23/2001 */
+#endif
          for(;;) {
             /* emit color in rgb 3 char encoded form */
             for (j = 0; j < 3; ++j) {
@@ -1093,7 +1122,7 @@ docolors:
             if (++curc >= maxcolor)      /* quit if done last color */
                break;
             if(debugflag == 920)  /* lossless compression */
-               continue;   
+               continue;
             /* Next a P Branderhorst special, a tricky scan for smooth-shaded
                ranges which can be written as <nn> to compress .par file entry.
                Method used is to check net change in each color value over
@@ -1175,7 +1204,7 @@ static void put_filename(char *keyword,char *fname)
       }
 }
 
-#ifndef XFRACT
+#ifndef USE_VARARGS
 static void put_parm(char *parm,...)
 #else
 static void put_parm(va_alist)
@@ -1185,7 +1214,7 @@ va_dcl
    char *bufptr;
    va_list args;
 
-#ifndef XFRACT
+#ifndef USE_VARARGS
    va_start(args,parm);
 #else
    char * parm;
@@ -1539,7 +1568,7 @@ void showfreemem(void)
 }
 #endif
 
-edit_text_colors()
+int edit_text_colors()
 {
    int save_debugflag,save_lookatmouse;
    int row,col,bkgrd;
@@ -1628,15 +1657,18 @@ static int modes_changed;
 
 int select_video_mode(int curmode)
 {
-   static FCODE o_hdg2[]={"key...name......................xdot.ydot.colr.comment.................."};
+   static FCODE o_hdg2[]={"key...name.......................xdot..ydot.colr.comment.................."};
    static FCODE o_hdg1[]={"Select Video Mode"};
    char hdg2[sizeof(o_hdg2)];
    char hdg1[sizeof(o_hdg1)];
 
    int entnums[MAXVIDEOMODES];
    int attributes[MAXVIDEOMODES];
-   int i,j,k,ret;
+   int i,k,ret;
+#ifndef XFRACT
+   int j;
    int oldtabmode,oldhelpmode;
+#endif
 
    load_fractint_cfg(0);        /* load fractint.cfg to extraseg */
 
@@ -1693,7 +1725,7 @@ int select_video_mode(int curmode)
    tabmode = 0;
    helpmode = HELPVIDSEL;
    i = fullscreen_choice(CHOICEHELP,hdg1,hdg2,NULL,vidtbllen,NULL,attributes,
-                         1,16,72,i,format_vid_table,NULL,NULL,check_modekey);
+                         1,16,74,i,format_vid_table,NULL,NULL,check_modekey);
    tabmode = oldtabmode;
    helpmode = oldhelpmode;
    if (i == -1) {
@@ -1753,20 +1785,22 @@ void format_vid_table(int choice,char *buf)
               sizeof(videoentry));
    vidmode_keyname(videoentry.keynum,kname);
    biosflag = (char)((videoentry.dotmode % 100 == 1) ? 'B' : ' ');
-   sprintf(buf,"%-5s %-25s %4d %4d ",  /* 42 chars */
+   sprintf(buf,"%-5s %-25s %5d %5d ",  /* 44 chars */
            kname, videoentry.name, videoentry.xdots, videoentry.ydots);
    if((truecolorbits = videoentry.dotmode/1000) == 0)
-      sprintf(local_buf,"%s%3d",  /* 45 chars */
+      sprintf(local_buf,"%s%3d",  /* 47 chars */
            buf, videoentry.colors);
    else 
-      sprintf(local_buf,"%s%3s",  /* 45 chars */
-           buf, (truecolorbits == 3)?"64m":
-                (truecolorbits == 2)?"16k":
-                (truecolorbits == 1)?"15k":"???");
-   sprintf(buf,"%s%c %-25s",  /* 72 chars */
+      sprintf(local_buf,"%s%3s",  /* 47 chars */
+           buf, (truecolorbits == 4)?" 4g":
+                (truecolorbits == 3)?"16m":
+                (truecolorbits == 2)?"64k":
+                (truecolorbits == 1)?"32k":"???");
+   sprintf(buf,"%s%c %-25s",  /* 74 chars */
            local_buf, biosflag, videoentry.comment);
 }
 
+#ifndef XFRACT
 static int check_modekey(int curkey,int choice)
 {
    int i,j,k,ret;
@@ -1803,6 +1837,7 @@ static int check_modekey(int curkey,int choice)
       }
    return(ret);
 }
+#endif
 
 static int entcompare(VOIDCONSTPTR p1,VOIDCONSTPTR p2)
 {
@@ -1863,21 +1898,22 @@ static void update_fractint_cfg()
             j += 8;
             }
          buf[i] = 0;
-         if((truecolorbits = videoentry.dotmode/1000) == 0)
-            sprintf(colorsbuf,"%3d",videoentry.colors);
+         if((truecolorbits = vident.dotmode/1000) == 0)
+            sprintf(colorsbuf,"%3d",vident.colors);
          else 
             sprintf(colorsbuf,"%3s",
-               buf, (truecolorbits == 3)?"64m":
-                    (truecolorbits == 2)?"16k":
-                    (truecolorbits == 1)?"15k":"???");
-         fprintf(outfile,"%-4s,%s,%4x,%4x,%4x,%4x,%4d,%4d,%4d,%s,%s\n",
+                    (truecolorbits == 4)?" 4g":
+                    (truecolorbits == 3)?"16m":
+                    (truecolorbits == 2)?"64k":
+                    (truecolorbits == 1)?"32k":"???");
+         fprintf(outfile,"%-4s,%s,%4x,%4x,%4x,%4x,%4d,%5d,%5d,%s,%s\n",
                 kname,
                 buf,
                 vident.videomodeax,
                 vident.videomodebx,
                 vident.videomodecx,
                 vident.videomodedx,
-                vident.dotmode%1000,
+                vident.dotmode%1000, /* remove true-color flag, keep textsafe */
                 vident.xdots,
                 vident.ydots,
                 colorsbuf,
