@@ -6,12 +6,17 @@
 #include <ctype.h>
 #include <time.h>
 #include <malloc.h>
+
 #ifndef XFRACT
-#include <stdarg.h>
 #include <io.h>
+#endif
+
+#ifndef USE_VARARGS
+#include <stdarg.h>
 #else
 #include <varargs.h>
 #endif
+
 /*#ifdef __TURBOC__
 #include <dir.h>
 #endif  */
@@ -140,8 +145,8 @@ int putstringwrap(int *row,int col1,int col2,int color,char far *str,int maxrow)
     return(done);
 }
 
-#define rad_to_deg(x) ((x)*(180/PI)) /* most people "think" in degrees */
-#define deg_to_rad(x) ((x)*(PI/180))
+#define rad_to_deg(x) ((x)*(180.0/PI)) /* most people "think" in degrees */
+#define deg_to_rad(x) ((x)*(PI/180.0))
 /*
 convert corners to center/mag
 Rotation angles indicate how much the IMAGE has been rotated, not the
@@ -164,9 +169,9 @@ void cvtcentermag(double *Xctr, double *Yctr, LDBL *Magnification, double *Xmagf
    { /* no rotation or skewing, but stretching is allowed */
       Width  = xxmax - xxmin;
       Height = yymax - yymin;
-      *Xctr = (xxmin + xxmax)/2;
-      *Yctr = (yymin + yymax)/2;
-      *Magnification  = 2/Height;
+      *Xctr = (xxmin + xxmax)/2.0;
+      *Yctr = (yymin + yymax)/2.0;
+      *Magnification  = 2.0/Height;
       *Xmagfactor =  Height / (DEFAULTASPECT * Width);
       *Rotation = 0.0;
       *Skew = 0.0;
@@ -191,14 +196,14 @@ void cvtcentermag(double *Xctr, double *Yctr, LDBL *Magnification, double *Xmagf
       b = sqrt(b2);
    
       tmpa = acos((a2+b2-c2)/(2*a*b)); /* save tmpa for later use */
-      *Skew = 90 - rad_to_deg(tmpa);
+      *Skew = 90.0 - rad_to_deg(tmpa);
    
-      *Xctr = (xxmin + xxmax)/2;
-      *Yctr = (yymin + yymax)/2;
+      *Xctr = (xxmin + xxmax)*0.5;
+      *Yctr = (yymin + yymax)*0.5;
    
       Height = b * sin(tmpa);
    
-      *Magnification  = 2/Height; /* 1/(h/2) */
+      *Magnification  = 2.0/Height; /* 1/(h/2) */
       *Xmagfactor = Height / (DEFAULTASPECT * a);
    
       /* if vector_a cross vector_b is negative */
@@ -310,7 +315,7 @@ void cvtcentermagbf(bf_t Xctr, bf_t Yctr, LDBL *Magnification, double *Xmagfacto
    LDBL Width, Height;
    LDBL a, b; /* bottom, left, diagonal */
    LDBL a2, b2, c2; /* squares of above */
-   LDBL tmpx1, tmpx2, tmpy, tmpy1, tmpy2 ;
+   LDBL tmpx1, tmpx2, tmpy=0.0, tmpy1, tmpy2 ;
    double tmpa; /* temporary x, y, angle */
    bf_t bfWidth, bfHeight;
    bf_t bftmpx, bftmpy;
@@ -733,7 +738,11 @@ static FCODE sflag_is_activated[] = {" flag is activated"};
 static FCODE sinteger_math[]      = {"Integer math is in use"};
 static FCODE sin_use_required[] = {" in use (required)"};
 static FCODE sarbitrary_precision[] = {"Arbitrary precision "};
+#ifdef XFRACT
+static FCODE spressanykey[] = {"Press any key to continue, F6 for area, F7 for next page"};
+#else
 static FCODE spressanykey[] = {"Press any key to continue, F6 for area, CTRL-TAB for next page"};
+#endif
 static FCODE spressanykey1[] = {"Press Esc to continue, Backspace for first screen"};
 static FCODE sbatch[] = {" (Batch mode)"};
 static FCODE ssavename[] = {"Savename: "};
@@ -774,7 +783,7 @@ int tab_display_2(char *msg)
    putstringcenter(s_row++,0,80,C_PROMPT_HI, sstopsecret);
    sprintf(msg,"Version %d patch %d",release, patchlevel);
    putstring(++s_row,2,C_GENERAL_HI,msg);
-   sprintf(msg,"%u bytes conventional stack free",stackavail());
+   sprintf(msg,"%lu bytes conventional stack free",stackavail());
    putstring(++s_row,2,C_GENERAL_HI,msg);
    sprintf(msg,"%ld of %ld bignum memory used",maxptr,maxstack);
    putstring(++s_row,2,C_GENERAL_HI,msg);
@@ -855,7 +864,7 @@ int tab_display_2(char *msg)
    putstring(s_row++,2,C_GENERAL_HI,msg);
    putstringcenter(24,0,80,C_GENERAL_LO,spressanykey1);
    *msg = 0;
-again: 
+again:
    putstring(s_row,2,C_GENERAL_HI,msg);
    key=getakeynohelp();
    if(key != ESC && key != BACKSPACE && key != TAB)
@@ -880,6 +889,10 @@ int tab_display()       /* display the status of the current image */
    int key;
    int saved=0;
    int dec;
+   int k;
+   U16 save_extra_handle = 0;
+   BYTE far *ptr_to_extraseg = NULL;
+   int hasformparam = 0;
 
    if (calc_status < 0) {       /* no active fractal image */
       return(0);                /* (no TAB on the credits screen) */
@@ -889,11 +902,23 @@ int tab_display()       /* display the status of the current image */
    stackscreen();
    if(bf_math)
    {
+   /* Save memory from the beginning of extraseg to ENDVID=22400 */
+   /* This is so the bf_math manipulations here don't corrupt */
+   /* the video modes or screen prompts. */
+      ptr_to_extraseg = MK_FP(extraseg,0);
+      save_extra_handle = MemoryAlloc((U16)22400, 1L, FARMEM);
+      MoveToMemory(ptr_to_extraseg,(U16)22400,1L,0L,save_extra_handle);
       saved = save_stack();
       bfXctr = alloc_stack(bflength+2);
       bfYctr = alloc_stack(bflength+2);
    }
+   if (fractype == FORMULA || fractype == FFORMULA)
+      for (i = 0; i < MAXPARAMS; i += 2)
+          if (!paramnotused(i))
+             hasformparam++;
 top:
+   k = 0; /* initialize here so parameter line displays correctly on return
+             from control-tab */
    helptitle();
    setattr(1,0,C_GENERAL_MED,24*80); /* init rest to background */
    s_row = 2;
@@ -1030,17 +1055,17 @@ top:
          case 4:
             putstring(s_row,2,C_GENERAL_HI,stesseral);
             break;
-	 case 5:
-	    putstring(s_row,2,C_GENERAL_HI,sdiffusion);
-	    break;
+         case 5:
+            putstring(s_row,2,C_GENERAL_HI,sdiffusion);
+            break;
          }
       ++s_row;
       if (got_status == 5 ) {
-	 sprintf(msg,"%2.2f%% done, counter at %lu of %lu (%u bits)",
-		     (100.0 * dif_counter)/dif_limit,
-		     dif_counter,dif_limit,bits);
-	 putstring(s_row,2,C_GENERAL_MED,msg);
-	 ++s_row;
+         sprintf(msg,"%2.2f%% done, counter at %lu of %lu (%u bits)",
+                 (100.0 * dif_counter)/dif_limit,
+                 dif_counter,dif_limit,bits);
+         putstring(s_row,2,C_GENERAL_MED,msg);
+         ++s_row;
       } else
       if (got_status != 3) {
          sprintf(msg,"Working on block (y,x) [%d,%d]...[%d,%d], ",
@@ -1175,20 +1200,22 @@ top:
 
    }
    }
-   for (i = 0; i < MAXPARAMS; i++)
+
+   if(typehasparm(fractype,0,msg) || hasformparam)
+    for (i = 0; i < MAXPARAMS; i++)
    {
       int col;
       char p[50];
-      if(i%4 == 0)
-      {
-         s_row++;
-         col = 9;
-      }
-      else
-         col = -1;
       if(typehasparm(fractype,i,p))
       {
-         if(i==0)
+         if(k%4 == 0)
+         {
+            s_row++;
+            col = 9;
+         }
+         else
+            col = -1;
+         if(k == 0) /* only true with first displayed parameter */
             putstring(++s_row,2,C_GENERAL_MED,sparams);
          sprintf(msg,"%3d: ",i+1);
          putstring(s_row,col,C_GENERAL_MED,msg);
@@ -1199,6 +1226,7 @@ top:
          else
             sprintf(msg,"%-12.9f",param[i]);
          putstring(-1,-1,C_GENERAL_HI,msg);
+         k++;
       }
    }
    putstring(s_row+=2,2,C_GENERAL_MED,siteration_maximum);
@@ -1250,7 +1278,12 @@ top:
    unstackscreen();
    timer_start = clock_ticks(); /* tab display was "time out" */
    if(bf_math)
+   {
       restore_stack(saved);
+      MoveFromMemory(ptr_to_extraseg,(U16)22400,1L,0L,save_extra_handle);
+      MemoryRelease(save_extra_handle);
+      save_extra_handle = 0;
+   }
    return(0);
 }
 
