@@ -19,18 +19,14 @@ extern void cdecl calcmandfpasmstart(void);
 /* extern long  cdecl calcmandfpasm(void); */
 extern long  cdecl calcmandfpasm_287(void);
 extern long  cdecl calcmandfpasm_87(void);
-#ifndef XFRACT
 extern long (*calcmandfpasm)(void);
-#else
-extern long calcmandfpasm(void);
-#endif
 
-#ifndef XFRACT
 /*  calmanp5 -- assembler file prototypes */
-
+#ifdef XFRACT
+extern long  cdecl calcmandfpasm_c(void);
+#endif
 extern long  cdecl calcmandfpasm_p5(void);
 extern void cdecl calcmandfpasmstart_p5(void);
-#endif
 
 /*  fpu087 -- assembler file prototypes */
 
@@ -182,8 +178,9 @@ extern unsigned long ReadTPlusBankedPixel(int, int);
 /*  video -- assembler file prototypes */
 
 extern void   cdecl adapter_detect(void);
-extern void   cdecl clearbox(void);
-extern void   cdecl dispbox(void);
+extern void   cdecl scroll_center(int, int);
+extern void   cdecl scroll_relative(int, int);
+extern void   cdecl scroll_state(int);
 extern void   cdecl setvideotext(void);
 extern void   cdecl setnullvideo(void);
 extern void   cdecl setfortext(void);
@@ -346,6 +343,7 @@ extern void displayc(int, int, int, int, int);
 
 extern int savetodisk(char *);
 extern int encoder(void);
+extern int _fastcall new_to_old(int new_fractype);
 
 /*  evolve -- C file prototypes */
 
@@ -354,11 +352,12 @@ extern  void param_history(int);
 extern  int get_variations(void);
 extern  int get_evolve_Parms(void);
 extern  void set_current_params(void);
-extern  void fiddleparms(GENEBASE gene[]);
+extern  void fiddleparms(GENEBASE gene[], int ecount);
 extern  void set_evolve_ranges(void);
 extern  void set_mutation_level(int);
 extern  void drawparmbox(int);
 extern  void spiralmap(int);
+extern  int unspiralmap(void);
 extern  int explore_check(void);
 extern  void SetupParamBox(void);
 extern  void ReleaseParamBox(void);
@@ -372,7 +371,7 @@ extern int t16_getline(FILE *,int ,U16 *);
 
 extern void calcfracinit(void);
 extern void adjust_corner(void);
-#ifndef XFRACT
+#ifndef USE_VARARGS
 extern int put_resume(int ,... );
 extern int get_resume(int ,... );
 #else
@@ -414,6 +413,7 @@ extern void close_snd(void);
 /*  fractalp -- C file prototypes */
 
 extern int typehasparm(int,int,char *);
+extern int paramnotused(int);
 
 /*  fractals -- C file prototypes */
 
@@ -591,7 +591,11 @@ extern void set_pixel_calc_functions(void);
 
 /*  fractint -- C file prototypes */
 
+#ifdef XFRACT
+extern int main(int argc,char **argv );
+#else
 extern void main(int argc,char **argv );
+#endif
 extern int elapsed_time(int);
 
 /*  framain2 -- C file prototypes */
@@ -605,12 +609,15 @@ extern int pot_line(BYTE *,int );
 extern int sound_line(BYTE *,int );
 extern int sound_line(unsigned char *pixels,int linelen);
 #ifndef XFRACT
-extern int timer(int,int (*subrtn)(),...);
 extern int _cdecl _matherr(struct exception *);
 #else
-extern int timer();
 extern int XZoomWaiting;
 #endif
+#ifndef USE_VARARGS
+extern int timer(int,int (*subrtn)(),...);
+#else
+extern int timer();
+#endif
 
 
 extern void clear_zoombox(void);
@@ -1041,9 +1048,9 @@ extern void NotAFnct(void );
 extern void FnctNotFound(void );
 extern int whichfn(char *,int );
 extern int CvtStk(void);
+extern int fFormula(void );
 #ifndef XFRACT
 extern void (far *isfunct(char *,int ))(void );
-extern int fFormula(void );
 #else
 extern void (far *isfunct(char *,int ))();
 #endif
@@ -1158,7 +1165,7 @@ extern char *despace(char *str);
 
 extern void rotate(int );
 extern void save_palette(void);
-extern void load_palette(void );
+extern int load_palette(void );
 
 /*  slideshw -- C file prototypes */
 
@@ -1249,6 +1256,8 @@ extern void aspectratio_crop(float ,float );
 extern int init_pan_or_recalc(int );
 extern void _fastcall drawlines(struct coords, struct coords, int, int);
 extern void _fastcall addbox(struct coords);
+extern void clearbox(void);
+extern void dispbox(void);
 
 /*  fractalb.c -- C file prototypes */
 
@@ -1306,6 +1315,36 @@ extern uclock_t usec_clock(void);
 extern void restart_uclock(void);
 extern void wait_until(int index, uclock_t wait_time);
 
+#ifdef XFRACT
+/*
+ *   general.c -- C file prototypes
+ */
+extern int waitkeypressed(int);
+extern void fix_ranges(int *, int, int);
+extern void decode_evolver_info(struct evolution_info *, int);
+extern void decode_fractal_info(struct fractal_info *, int);
+
+/*
+ *   unix.c -- C file prototypes
+ */
+extern long clock_ticks(void);
+extern int stricmp(char *, char *);
+extern int strnicmp(char *, char *, int);
+extern int memicmp(char *, char *, int);
+extern unsigned short _rotl(unsigned short, short);
+extern int ltoa(long, char *, int);
+extern void ftimex(struct timebx *);
+extern long stackavail(void);
+extern int kbhit(void);
+
+/*
+ *   video.c -- C file prototypes
+ */
+extern void putprompt(void);
+extern void loaddac(void);
+
+#endif
+
 #ifndef DEBUG
 /*#define DEBUG */
 #endif
