@@ -6,6 +6,8 @@
 #ifndef XFRACT
 #include <io.h>
 #include <process.h>
+#else
+#include <curses.h>
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,11 +26,8 @@ static int menu_checkkey(int curkey,int choice);
 /*
 #define PRODUCTION
 */
-int release=2000;  /* this has 2 implied decimals; increment it every synch */
-int patchlevel=0; /* patchlevel for DOS version */
-#ifdef XFRACT
-int xrelease=310;
-#endif
+int release=2003;  /* this has 2 implied decimals; increment it every synch */
+int patchlevel=1; /* patchlevel for DOS version */
 
 /* fullscreen_choice options */
 #define CHOICERETURNKEY 1
@@ -59,7 +58,7 @@ static char far s_escape_cancel[] = {"Escape to cancel, any other key to continu
 static char far s_anykey[] = {"Any key to continue..."};
 #if !defined(PRODUCTION) && !defined(XFRACT)
 static char far s_custom[] = {"Customized Version"};
-static char far s_notpublic[] = {"Not for Public Release"};
+static char far s_incremental[] = {"Incremental release"};
 #endif
 int stopmsg (int flags, char far *msg)
 {
@@ -73,7 +72,7 @@ int stopmsg (int flags, char far *msg)
       else
          fp=dir_fopen(workdir,"stopmsg.txt","a");
       if(fp != NULL)
-#ifndef XRACT
+#ifndef XFRACT
       fprintf(fp,"%Fs\n",msg);
 #else
       fprintf(fp,"%s\n",msg);
@@ -209,7 +208,12 @@ int showtempmsg(char far *msgparm)
    size = (long)textxdots * (long)textydots;
    save_sxoffs = sxoffs;
    save_syoffs = syoffs;
-   sxoffs = syoffs = 0;
+   if (video_scroll) {
+      sxoffs = video_startx;
+      syoffs = video_starty;
+   }
+   else
+      sxoffs = syoffs = 0;
    if(temptextsave == 0) /* only save screen first time called */
    {
       if ((temptextsave = MemoryAlloc((U16)textxdots,(long)textydots,FARMEM)) == 0)
@@ -259,7 +263,12 @@ void cleartempmsg()
    else if (temptextsave != 0) {
       save_sxoffs = sxoffs;
       save_syoffs = syoffs;
-      sxoffs = syoffs = 0;
+      if (video_scroll) {
+         sxoffs = video_startx;
+         syoffs = video_starty;
+      }
+      else
+         sxoffs = syoffs = 0;
       for (i = 0; i < textydots; ++i) {
          MoveFromMemory(buffer,(U16)textxdots,1L,(long)i,temptextsave);
          put_line(i,0,textxdots-1,buffer);
@@ -293,33 +302,35 @@ void helptitle()
    char msg[MSGLEN],buf[MSGLEN];
    setclear(); /* clear the screen */
 #ifdef XFRACT
-   sprintf(msg,"XFRACTINT  Version %d.%02d (FRACTINT Version %d.%02d)",
-           xrelease/100,xrelease%100, release/100,release%100);
+   strcpy(msg,"X");
 #else
-   sprintf(msg,"FRACTINT Version %d.%01d",release/100,(release%100)/10);
+   *msg=0;
+#endif   
+   sprintf(buf,"FRACTINT Version %d.%01d",release/100,(release%100)/10);
+   strcat(msg,buf);
    if (release%10) {
       sprintf(buf,"%01d",release%10);
       strcat(msg,buf);
       }
-#endif
    if (patchlevel) {
-      sprintf(buf," P%d",patchlevel);
+      sprintf(buf,".%d",patchlevel);
       strcat(msg,buf);
       }
    putstringcenter(0,0,80,C_TITLE,msg);
+   
 /* uncomment next for production executable: */
 #if defined(PRODUCTION) || defined(XFRACT)
     return;
    /*NOTREACHED*/
 #else
    if (debugflag == 3002) return;
-/* #define DEVELOPMENT */
+#define DEVELOPMENT
 #ifdef DEVELOPMENT
    putstring(0,2,C_TITLE_DEV,"Development Version");
 #else
    putstring(0,3,C_TITLE_DEV, s_custom);
 #endif
-   putstring(0,55,C_TITLE_DEV,s_notpublic);
+   putstring(0,55,C_TITLE_DEV,s_incremental);
 #endif
 }
 
@@ -330,7 +341,7 @@ void footer_msg(int *i, int options, char *speedstring)
    static FCODE choiceinstr1b[]="Use the cursor keys or type a value to make a selection";
    static FCODE choiceinstr2a[]="Press ENTER for highlighted choice, or ESCAPE to back out";
    static FCODE choiceinstr2b[]="Press ENTER for highlighted choice, ESCAPE to back out, or F1 for help";
-   static FCODE choiceinstr2c[]="Press ENTER for highlighted choice, or F1 for help";
+   static FCODE choiceinstr2c[]="Press ENTER for highlighted choice, or "FK_F1" for help";
    putstringcenter((*i)++,0,80,C_PROMPT_BKGRD,
       (speedstring) ? choiceinstr1b : choiceinstr1a);
    putstringcenter(*(i++),0,80,C_PROMPT_BKGRD,
@@ -348,10 +359,11 @@ int putstringcenter(int row, int col, int width, int attr, char far *msg)
    int i,j,k;
    i = 0;
 #ifdef XFRACT
-   if (width==80) width=79; /* Some systems choke in column 80 */
+   if (width>=80) width=79; /* Some systems choke in column 80 */
 #endif
    while (msg[i]) ++i; /* strlen for a far */
    if (i == 0) return(-1);
+   if (i >= width) i = width - 1; /* sanity check */
    j = (width - i) / 2;
    j -= (width + 10 - i) / 20; /* when wide a bit left of center looks better */
    memset(buf,' ',width);
@@ -367,7 +379,7 @@ int putstringcenter(int row, int col, int width, int attr, char far *msg)
  * The stackscreen()/unstackscreen() functions were originally
  * ported to Xfractint. However, since Xfractint uses a separate
  * text and graphics window, I don't see what these functions
- * are good for, so I have commented them out. 
+ * are good for, so I have commented them out.
  *
  * To restore the Xfractint versions of these functions,
  * uncomment next.
@@ -381,6 +393,8 @@ int putstringcenter(int row, int col, int width, int attr, char far *msg)
 static int screenctr = -1;
 #else
 static int screenctr = 0;
+extern void savecurses(WINDOW **);
+extern void restorecurses(WINDOW **);
 #endif
 
 #define MAXSCREENS 3
@@ -398,6 +412,10 @@ void stackscreen()
    BYTE far *vidmem;
    int savebytes;
    int i;
+   if (video_scroll) {
+      scroll_state(0); /* save position */
+      scroll_center(0,0);
+   }
    if(*s_makepar == 0)
       return;
    saverc[screenctr+1] = textrow*80 + textcol;
@@ -421,6 +439,10 @@ void stackscreen()
       }
    else
       setfortext();
+   if (video_scroll) {
+      if (boxcount)
+         moveboxf(0.0,0.0);
+   }
 #elif defined (USE_XFRACT_STACK_FUNCTIONS)
    int i;
    BYTE far *ptr;
@@ -431,8 +453,8 @@ void stackscreen()
          stopmsg(1,msg);
          exit(1);
          }
-      if (ptr = savescreen[i] = farmemalloc(sizeof(int *)))
-         savecurses(ptr);
+      if ((ptr = (savescreen[i] = farmemalloc(sizeof(int *)))))
+         savecurses((WINDOW **)ptr);
       else {
          stopmsg(1,msg);
          exit(1);
@@ -465,13 +487,18 @@ void unstackscreen()
    else
       setforgraphics();
    movecursor(-1,-1);
+   if (video_scroll) {
+      if (boxcount)
+         moveboxf(0.0,0.0);
+      scroll_state(1); /* restore position */
+   }
 #elif defined (USE_XFRACT_STACK_FUNCTIONS)
    BYTE far *ptr;
    textrow = saverc[screenctr] / 80;
    textcol = saverc[screenctr] % 80;
    if (--screenctr >= 0) { /* unstack */
       ptr = savescreen[screenctr];
-      restorecurses(ptr);
+      restorecurses((WINDOW **)ptr);
       farmemfree(ptr);
       }
    else
@@ -560,7 +587,9 @@ void process_speedstring(char    *speedstring,
       speedstring[--i] = 0;
    if (33 <= curkey && curkey <= 126 && i < 30)
    {
+#ifndef XFRACT
       curkey = tolower(curkey);
+#endif
       speedstring[i] = (char)curkey;
       speedstring[++i] = 0;
    }
@@ -734,10 +763,19 @@ int fullscreen_choice(
             }
          }
       }
+#if 0
    if ((i = 77 / boxwidth - colwidth) > 3) /* spaces to add @ left each choice */
       i = 3;
    if (i == 0)
       i = 1;
+#else
+   if ((i = (80 / boxwidth - colwidth) / 2 - 1) == 0) /* to allow wider prompts */
+      i = 1;
+   if (i < 0)
+      i = 0;
+   if (i > 3)
+      i = 3;
+#endif
    j = boxwidth * (colwidth += i) + i;     /* overall width of box */
    if (j < titlewidth+2)
       j = titlewidth + 2;
@@ -1133,11 +1171,6 @@ top:
       choicekey[nextleft+=2] = 9; /* tab */
       attributes[nextleft] = MENU_ITEM;
       LOADPROMPTSCHOICES(nextleft,"info about image      <tab> ");
-/*  This is a help screen and doesn't belong here.
-      choicekey[nextleft+=2] = -10;
-      attributes[nextleft] = MENU_ITEM;
-      LOADPROMPTSCHOICES(nextleft,"zoom box functions...       ");
-*/
       choicekey[nextleft+=2] = 'o';
       attributes[nextleft] = MENU_ITEM;
       LOADPROMPTSCHOICES(nextleft,"orbits window          <o>  ");
@@ -1146,9 +1179,15 @@ top:
       }
    LOADPROMPTSCHOICES(nextleft+=2,"      NEW IMAGE             ");
    attributes[nextleft] = 256+MENU_HDG;
+#ifdef XFRACT
+   choicekey[nextleft+=2] = DELETE;
+   attributes[nextleft] = MENU_ITEM;
+   LOADPROMPTSCHOICES(nextleft,"draw fractal           <D>  ");
+#else
    choicekey[nextleft+=2] = DELETE;
    attributes[nextleft] = MENU_ITEM;
    LOADPROMPTSCHOICES(nextleft,"select video mode...  <del> ");
+#endif
    choicekey[nextleft+=2] = 't';
    attributes[nextleft] = MENU_ITEM;
    LOADPROMPTSCHOICES(nextleft,"select fractal type    <t>  ");
@@ -1240,19 +1279,29 @@ top:
       attributes[nextright] = MENU_ITEM;
       LOADPROMPTSCHOICES(nextright,"print image              <p>  ");
       }
+#ifdef XFRACT
+   choicekey[nextright+=2] = 'd';
+   attributes[nextright] = MENU_ITEM;
+   LOADPROMPTSCHOICES(nextright,"shell to Linux/Unix      <d>  ");
+#else
    choicekey[nextright+=2] = 'd';
    attributes[nextright] = MENU_ITEM;
    LOADPROMPTSCHOICES(nextright,"shell to dos             <d>  ");
+#endif
    choicekey[nextright+=2] = 'g';
    attributes[nextright] = MENU_ITEM;
    LOADPROMPTSCHOICES(nextright,"give command string      <g>  ");
    choicekey[nextright+=2] = ESC;
    attributes[nextright] = MENU_ITEM;
-   LOADPROMPTSCHOICES(nextright,"quit Fractint           <esc> ");
+   LOADPROMPTSCHOICES(nextright,"quit "FRACTINT"           <esc> ");
    choicekey[nextright+=2] = INSERT;
    attributes[nextright] = MENU_ITEM;
-   LOADPROMPTSCHOICES(nextright,"restart Fractint        <ins> ");
+   LOADPROMPTSCHOICES(nextright,"restart "FRACTINT"        <ins> ");
+#ifdef XFRACT
+   if (fullmenu && (gotrealdac || fake_lut) && colors >= 16) {
+#else
    if (fullmenu && gotrealdac && colors >= 16) {
+#endif
       /* nextright += 2; */
       LOADPROMPTSCHOICES(nextright+=2,"       COLORS                 ");
       attributes[nextright] = 256+MENU_HDG;
@@ -1267,19 +1316,19 @@ top:
             choicekey[nextright+=2] = 'e';
             attributes[nextright] = MENU_ITEM;
             LOADPROMPTSCHOICES(nextright,"palette editing mode     <e>  ");
-            }
+         }
          choicekey[nextright+=2] = 'a';
          attributes[nextright] = MENU_ITEM;
          LOADPROMPTSCHOICES(nextright,"make starfield           <a>  ");
-         }
-      choicekey[nextright+=2] = 1;
-      attributes[nextright] = MENU_ITEM;
-      LOADPROMPTSCHOICES(nextright,   "ant automaton          <ctl-a>");
-
-      choicekey[nextright+=2] = 19;
-      attributes[nextright] = MENU_ITEM;
-      LOADPROMPTSCHOICES(nextright,   "stereogram             <ctl-s>");
       }
+   }
+   choicekey[nextright+=2] = 1;
+   attributes[nextright] = MENU_ITEM;
+   LOADPROMPTSCHOICES(nextright,   "ant automaton          <ctl-a>");
+
+   choicekey[nextright+=2] = 19;
+   attributes[nextright] = MENU_ITEM;
+   LOADPROMPTSCHOICES(nextright,   "stereogram             <ctl-s>");
 
    i = (keypressed()) ? getakey() : 0;
    if (menu_checkkey(i,0) == 0) {
@@ -1306,7 +1355,11 @@ top:
          }
       }
    if (i == ESC) {             /* escape from menu exits Fractint */
+#ifdef XFRACT
+      static char far s[] = "Exit from Xfractint (y/n)? y";
+#else
       static char far s[] = "Exit from Fractint (y/n)? y";
+#endif
       helptitle();
       setattr(1,0,C_GENERAL_MED,24*80);
       for (i = 9; i <= 11; ++i)
@@ -1344,11 +1397,11 @@ static int menu_checkkey(int curkey,int choice)
 #endif
    if(testkey == '2')
       testkey = '@';
-   if (strchr("#@2txyzgvir3dj",testkey) || testkey == INSERT
+   if (strchr("#@2txyzgvir3dj",testkey) || testkey == INSERT
      || testkey == ESC || testkey == DELETE || testkey ==6) /*RB 6== ctrl-F for sound menu */
       return(0-testkey);
    if (menutype) {
-      if (strchr("\\sobpkrh",testkey) || testkey == TAB)
+      if (strchr("\\sobpkrh",testkey) || testkey == TAB)
          return(0-testkey);
       if (testkey == ' ')
          if ((curfractalspecific->tojulia != NOFRACTAL
@@ -1652,7 +1705,6 @@ int savegraphics()
    long count;
    unsigned long swaptmpoff;
 
-   discardgraphics();
    swaptotlen = (long)(vxdots > sxdots ? vxdots : sxdots) * (long)sydots;
    i = colors;
    while (i <= 16) {
@@ -1784,6 +1836,9 @@ int load_fractint_cfg(int options)
    vident = vidtbl;
    while (vidtbllen < MAXVIDEOMODES
      && fgets(tempstring, 120, cfgfile)) {
+      if(strchr(tempstring,'\n') == NULL)
+         /* finish reading the line */
+         while(fgetc(cfgfile) != '\n' && !feof(cfgfile));
       ++linenum;
       if (tempstring[0] == ';') continue;   /* comment line */
       tempstring[120] = 0;
@@ -1809,10 +1864,15 @@ int load_fractint_cfg(int options)
       xdots       = atol(&tempstring[commas[6]]);
       ydots       = atol(&tempstring[commas[7]]);
       colors      = atoi(&tempstring[commas[8]]);
-      if(colors == 16 && strchr(strlwr(&tempstring[commas[8]]),'m'))
+      if(colors == 4 && strchr(strlwr(&tempstring[commas[8]]),'g'))
       {
          colors = 256;
-         truecolorbits = 3; /* 48 bits */
+         truecolorbits = 4; /* 32 bits */
+      }
+      else if(colors == 16 && strchr(&tempstring[commas[8]],'m'))
+      {
+         colors = 256;
+         truecolorbits = 3; /* 24 bits */
       }
       else if(colors == 64 && strchr(&tempstring[commas[8]],'k'))
       {
@@ -1829,7 +1889,7 @@ int load_fractint_cfg(int options)
 
       textsafe2   = dotmode / 100;
       dotmode    %= 100;
-      if (j != 9 ||
+      if (j < 9 ||
             keynum < 0 ||
             dotmode < 0 || dotmode > 30 ||
             textsafe2 < 0 || textsafe2 > 4 ||
@@ -1913,7 +1973,7 @@ int check_vidmode_key(int option,int k)
    /* function key currently assigned to a video mode, -1 otherwise */
    if (k == 1400)              /* special value from select_vid_mode  */
       return(MAXVIDEOTABLE-1); /* for last entry with no key assigned */
-   if (k != 0)
+   if (k != 0) {
       if (option == 0) { /* check resident video mode table */
          for (i = 0; i < MAXVIDEOTABLE; ++i) {
             if (videotable[i].keynum == k)
@@ -1926,6 +1986,7 @@ int check_vidmode_key(int option,int k)
                return(i);
             }
          }
+   }
    return(-1);
 }
 
@@ -1991,4 +2052,3 @@ void vidmode_keyname(int k,char *buf)
 #if (_MSC_VER >= 700)
 #pragma code_seg ()      /* back to normal segment */
 #endif
-
