@@ -15,12 +15,15 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <curses.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
+#include <X11/Xutil.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #ifdef _AIX
 #include <sys/select.h>
 #endif
@@ -43,6 +46,7 @@
 # include <sys/filio.h>
 # define FNDELAY O_NONBLOCK
 #endif
+#include <assert.h>
 
 /* Check if there is a character waiting for us.  */
 #define input_pending() (ioctl(0,FIONREAD,&iocount),(int)iocount)
@@ -82,9 +86,12 @@ static int fullscreen = 0;
 static int sharecolor = 0;
 static int privatecolor = 0;
 static int fixcolors = 0;
-static int sync = 0; /* Run X events synchronously (debugging) */
+static int synch = 0; /* Run X events synchronously (debugging) */
 int slowdisplay = 0; /* We have a slow display, so don't print too much */
 static int simple_input = 0; /* Use simple input (debugging) */
+static int resize_flag = 0; /* Main window being resized ? */
+static int drawing_or_drawn = 0; /* Is image (being) drawn ? */
+
 static char *Xdisplay = "";
 static char *Xgeometry = NULL;
 
@@ -99,8 +106,16 @@ static int doesBacking;
  * the actual color table entries which may be anything.
  */
 static int usepixtab = 0;
-static unsigned long pixtab[256];
 static int ipixtab[256];
+static unsigned long pixtab[256];
+typedef unsigned long XPixel;
+
+static XPixel cmap_pixtab[256]; /* for faking a LUTs on non-LUT visuals */
+static int cmap_pixtab_alloced;
+static unsigned long do_fake_lut(int idx) {
+  return fake_lut ? cmap_pixtab[idx] : idx;
+}
+#define FAKE_LUT(idx_) do_fake_lut(idx_)
 
 static int fastmode = 0; /* Don't draw pixels 1 at a time */
 static int alarmon = 0; /* 1 if the refresh alarm is on */
@@ -114,7 +129,7 @@ static int getachar(void);
 static int handleesc(void); 
 static int translatekey(int ch); 
 static int xcmapstuff(void); 
-static int xhandleevents(void); 
+static void xhandleevents(void);
 static void RemoveRootPixmap(void);
 static void doneXwindow(void);
 static void initdacbox(void);
@@ -179,7 +194,7 @@ int *i;
 	slowdisplay = 1;
 	return 1;
     } else if (strcmp(argv[*i],"-sync")==0) {
-	sync = 1;
+	synch = 1;
 	return 1;
     } else if (strcmp(argv[*i],"-private")==0) {
 	privatecolor = 1;
@@ -241,7 +256,7 @@ UnixInit()
     initdacbox();
 
     if (!simple_input) {
-	  signal(SIGINT,(__sighandler_t)goodbye);
+	  signal(SIGINT,(SignalHandler)goodbye);
     }
     signal(SIGFPE, fpe_handler);
     /*
@@ -315,6 +330,7 @@ XErrorEvent *xe;
 		xe->request_code, xe->minor_code);
         XGetErrorText(dp,xe->error_code,buf,200);
         printf("%s\n",buf);
+        return(0);
 }
 
 #ifdef FPUERR
@@ -364,15 +380,64 @@ static Screen *Xsc;
 static Colormap	Xcmap;
 static int Xdepth;
 static XImage *Ximage =NULL;
-static char *Xdata;
 static int Xdscreen;
 static Pixmap	Xpixmap = 0;
 static int Xwinwidth=DEFX,Xwinheight=DEFY;
+static XSizeHints *size_hints = NULL;
+static int gravity;
 static Window Xroot;
 static int xlastcolor = -1;
 static int xlastfcn = GXcopy;
 static BYTE *pixbuf = NULL;
 
+static int step = 0;
+static int cyclic[][3] = { 
+{1,3,5}, {1,5,3}, {3,1,5}, {3,5,1}, {5,1,3}, {5,3,1}, 
+{1,3,7}, {1,7,3}, {3,1,7}, {3,7,1}, {7,1,3}, {7,3,1}, 
+{1,5,7}, {1,7,5}, {5,1,7}, {5,7,1}, {7,1,5}, {7,5,1}, 
+{3,5,7}, {3,7,5}, {5,3,7}, {5,7,3}, {7,3,5}, {7,5,3}
+};
+
+static void
+select_visual(void)
+{
+  Xvi = XDefaultVisualOfScreen(Xsc);
+  Xdepth = DefaultDepth(Xdp, Xdscreen);
+
+  switch (Xvi->class) {
+  case StaticGray:
+  case StaticColor:
+    colors = 1 << Xdepth;
+    gotrealdac = 0;
+    fake_lut = 0;
+    istruecolor = 0;
+    break;
+
+  case GrayScale:
+  case PseudoColor:
+    colors = 1 << Xdepth;
+    gotrealdac = 1;
+    fake_lut = 0;
+    istruecolor = 0;
+    break;
+
+  case TrueColor:
+  case DirectColor:
+    colors = 256;
+    gotrealdac = 0;
+    fake_lut = 1;
+    istruecolor = 0;
+    break;
+
+  default:
+    /* those should be all the visual classes */
+    assert(1);
+    break;
+  }
+  if (colors > 256)
+    colors = 256;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -392,139 +457,137 @@ static BYTE *pixbuf = NULL;
 void
 initUnixWindow()
 {
-    XSetWindowAttributes Xwatt;
-    XGCValues Xgcvals;
-    int Xwinx=0,Xwiny=0;
-    int i;
+  XSetWindowAttributes Xwatt;
+  XGCValues Xgcvals;
+  int Xwinx = 0, Xwiny = 0;
+  int i;
 
-    if (Xdp != NULL) {
-       /* We are already initialized */
-       return;
+  if (Xdp != NULL) {
+    /* We are already initialized */
+    return;
+  }
+
+  if (!simple_input) {
+    old_fcntl = fcntl(0, F_GETFL);
+    fcntl(0, F_SETFL, FNDELAY);
+  }
+  
+  adapter = 0;
+
+  /* We have to do some X stuff even for disk video, to parse the geometry
+   * string */
+
+  if (unixDisk) {
+    int offx, offy;
+    fastmode = 0;
+    fake_lut = 0;
+    istruecolor = 0;
+    gotrealdac = 1;
+    colors = 256;
+    for (i = 0; i < colors; i++) {
+      pixtab[i] = i;
+      ipixtab[i] = i;
+    }
+    if (fixcolors > 0) {
+      colors = fixcolors;
+    }
+    if (Xgeometry) {
+      XParseGeometry(Xgeometry, &offx, &offy, (unsigned int *) &Xwinwidth,
+		     (unsigned int *) &Xwinheight);
+    }
+    Xwinwidth &= -4;
+    Xwinheight &= -4;
+    sxdots = Xwinwidth;
+    sydots = Xwinheight;
+  } else {  /* Use X window */
+    size_hints = XAllocSizeHints();
+    if (size_hints == NULL) {
+       fprintf(stderr, "Could not allocate memory for X size hints \n");
+       fprintf(stderr, "Note: xfractint can run without X in -disk mode\n");
+       UnixDone();
+       exit(-1);
     }
 
-    if (!simple_input) {
-	old_fcntl = fcntl(0,F_GETFL);
-	fcntl(0,F_SETFL,FNDELAY);
+    if (Xgeometry) {
+       size_hints->flags = USSize | PBaseSize | PResizeInc;
+    }
+    else {
+       size_hints->flags = PSize | PBaseSize | PResizeInc;
+    }
+    size_hints->base_width = 320;
+    size_hints->base_height = 200;
+    size_hints->width_inc = 4;
+    size_hints->height_inc = 4;
+  
+    Xdp = XOpenDisplay(Xdisplay);
+    if (Xdp == NULL) {
+      fprintf(stderr, "Could not open display %s\n", Xdisplay);
+      fprintf(stderr, "Note: xfractint can run without X in -disk mode\n");
+      UnixDone();
+      exit(-1);
+    }
+    Xdscreen = XDefaultScreen(Xdp);
+    if (Xgeometry && !onroot) {
+      int offx, offy;
+      XParseGeometry(Xgeometry, &offx, &offy, (unsigned int *) &Xwinwidth,
+		     (unsigned int *) &Xwinheight);
+      XWMGeometry(Xdp, Xdscreen, Xgeometry, DEFXY, 0, size_hints,
+		&Xwinx, &Xwiny, &Xwinwidth, &Xwinheight, &gravity);
+    }
+    if (synch) {
+      XSynchronize(Xdp, True);
+    }
+    XSetErrorHandler(errhand);
+    Xsc = ScreenOfDisplay(Xdp, Xdscreen);
+    select_visual();
+    if (fixcolors > 0) {
+      colors = fixcolors;
     }
 
-    /*
-    initmode = 0;
-    */
-    adapter = 0;
+    if (fullscreen || onroot) {
+      Xwinwidth = DisplayWidth(Xdp, Xdscreen);
+      Xwinheight = DisplayHeight(Xdp, Xdscreen);
+    }
+    sxdots = Xwinwidth;
+    sydots = Xwinheight;
 
-    /* We have to do some X stuff even for disk video, to parse the geometry
-     * string */
-
-    if (unixDisk) {
-	int offx, offy;
-	fastmode = 0;
-	gotrealdac=1;
-	colors = 256;
-	for (i=0;i<colors;i++) {
-	    pixtab[i] = i;
-	    ipixtab[i] = i;
-	}
-	if (fixcolors>0) {
-	    colors = fixcolors;
-	}
-	if (Xgeometry) {
-	    XParseGeometry(Xgeometry, &offx, &offy, (unsigned int *)&Xwinwidth,
-		    (unsigned int *)&Xwinheight);
-	}
-	sxdots = Xwinwidth;
-	sydots = Xwinheight;
+    Xwatt.background_pixel = BlackPixelOfScreen(Xsc);
+    Xwatt.bit_gravity = StaticGravity;
+    doesBacking = DoesBackingStore(Xsc);
+    if (doesBacking) {
+      Xwatt.backing_store = Always;
     } else {
-	Xdp = XOpenDisplay(Xdisplay);
-	if (Xdp==NULL) {
-	    fprintf(stderr,"Could not open display %s\n",Xdisplay);
-	    fprintf(stderr,"Note: xfractint can run without X in -disk mode\n");
-	    UnixDone();
-	    exit(-1);
-	}
-	Xdscreen = XDefaultScreen(Xdp);
-	if (Xgeometry && !onroot) {
-	    XGeometry(Xdp, Xdscreen, Xgeometry, DEFXY, 0, 1, 1, 0, 0,
-		    &Xwinx, &Xwiny, &Xwinwidth, &Xwinheight);
-	}
-	if (sync) {
-	    XSynchronize(Xdp,True);
-	}
-	XSetErrorHandler(errhand);
-	Xsc = ScreenOfDisplay(Xdp,Xdscreen);
-	Xvi = XDefaultVisualOfScreen(Xsc);
-	Xdepth = DefaultDepth(Xdp,Xdscreen);
-	colors = 1<<Xdepth;
-	if (colors>256) colors = 256;
-	if (fixcolors>0) {
-	    colors = fixcolors;
-	}
-	switch (Xvi->class) {
-	    case DirectColor:
-	    case PseudoColor:
-	    case GrayScale:
-		gotrealdac=1;
-		break;
-	    case TrueColor:
-	    case StaticColor:
-	    case StaticGray:
-	    default:
-		gotrealdac=0;
-		break;
-	}
-
-	if (fullscreen || onroot) {
-	    Xwinwidth = DisplayWidth(Xdp,Xdscreen);
-	    Xwinheight = DisplayHeight(Xdp,Xdscreen);
-	}
-	sxdots = Xwinwidth;
-	sydots = Xwinheight;
-
-	Xwatt.background_pixel = BlackPixelOfScreen(Xsc);
-	Xwatt.bit_gravity = StaticGravity;
-	doesBacking = DoesBackingStore(Xsc);
-	if (doesBacking) {
-	    Xwatt.backing_store = Always;
-	} else {
-	    Xwatt.backing_store = NotUseful;
-	}
-	if (onroot) {
-	    Xroot = FindRootWindow();
-	    RemoveRootPixmap();
-	    /*
-	    XSetWindowBackground(Xdp,Xroot);
-	    */
-	    Xgc = XCreateGC(Xdp,Xroot,0,&Xgcvals);
-	    Xpixmap = XCreatePixmap(Xdp,Xroot,Xwinwidth,Xwinheight,Xdepth);
-	    Xw = Xroot;
-	    XFillRectangle(Xdp,Xpixmap,Xgc,0,0,Xwinwidth,Xwinheight);
-	    XSetWindowBackgroundPixmap(Xdp,Xroot,Xpixmap);
-	} else {
-	    Xroot = DefaultRootWindow(Xdp);
-	    Xw = XCreateWindow(Xdp, Xroot, Xwinx, Xwiny, Xwinwidth,
-		    Xwinheight, 0, Xdepth, InputOutput, CopyFromParent,
-		    CWBackPixel|CWBitGravity|CWBackingStore, &Xwatt);
-	    XStoreName(Xdp,Xw,"xfractint");
-	    Xgc = XCreateGC(Xdp,Xw,0,&Xgcvals);
-	}
-	colors = xcmapstuff();
-	if (rotate_hi==255) rotate_hi = colors-1;
-	if (!onroot) {
-	    XSetBackground(Xdp,Xgc,pixtab[0]);
-	    XSetForeground(Xdp,Xgc,pixtab[1]);
-	    Xwatt.background_pixel = pixtab[0];
-	    XChangeWindowAttributes(Xdp,Xw,CWBackPixel,&Xwatt);
-	    XMapWindow(Xdp,Xw);
-	}
-	if (onroot) {
-	    XSelectInput(Xdp,Xw,KeyPressMask|KeyReleaseMask|ExposureMask);
-	} else {
-	    XSelectInput(Xdp,Xw,KeyPressMask|KeyReleaseMask|ExposureMask|
-		    ButtonPressMask|ButtonReleaseMask|PointerMotionMask);
-	}
-
-	resizeWindow();
-	xsync();
+      Xwatt.backing_store = NotUseful;
     }
+    if (onroot) {
+      Xroot = FindRootWindow();
+      RemoveRootPixmap();
+      Xgc = XCreateGC(Xdp, Xroot, 0, &Xgcvals);
+      Xpixmap = XCreatePixmap(Xdp, Xroot, Xwinwidth, Xwinheight, Xdepth);
+      Xw = Xroot;
+      XFillRectangle(Xdp, Xpixmap, Xgc, 0, 0, Xwinwidth, Xwinheight);
+      XSetWindowBackgroundPixmap(Xdp, Xroot, Xpixmap);
+    } else {
+      Xroot = DefaultRootWindow(Xdp);
+      Xw = XCreateWindow(Xdp, Xroot, Xwinx, Xwiny, Xwinwidth,
+			 Xwinheight, 0, Xdepth, InputOutput, CopyFromParent,
+			 CWBackPixel | CWBitGravity | CWBackingStore, &Xwatt);
+      XStoreName(Xdp, Xw, Fractint);
+      Xgc = XCreateGC(Xdp, Xw, 0, &Xgcvals);
+    }
+    colors = xcmapstuff();
+    if (rotate_hi == 255) rotate_hi = colors-1;
+
+    XSetWMNormalHints(Xdp, Xw, size_hints);
+
+    if (!onroot) {
+      XSetBackground(Xdp, Xgc, FAKE_LUT(pixtab[0]));
+      XSetForeground(Xdp, Xgc, FAKE_LUT(pixtab[1]));
+      Xwatt.background_pixel = FAKE_LUT(pixtab[0]);
+      XChangeWindowAttributes(Xdp, Xw, CWBackPixel, &Xwatt);
+      XMapWindow(Xdp, Xw);
+    }
+  }
 
     writevideopalette();
 
@@ -562,6 +625,9 @@ doneXwindow()
 	Xpixmap = (Pixmap)NULL;
     }
     XFlush(Xdp);
+    if (size_hints) {
+       XFree(size_hints);
+    }
     /*
     XCloseDisplay(Xdp);
     */
@@ -586,38 +652,43 @@ doneXwindow()
 static void
 clearXwindow()
 {
-    char *ptr;
-    int i,len;
-    if (pixtab[0] != 0) {
-	/*
-	 * Initialize image to pixtab[0].
-	 */
-	if (colors==2) {
-	    for (i=0;i<Ximage->bytes_per_line;i++) {
-		Ximage->data[i] = 0xff;
-	    }
-	} else {
-	    for (i=0;i<Ximage->bytes_per_line;i++) {
-		Ximage->data[i] = pixtab[0];
-	    }
-	}
-	for (i=1;i<Ximage->height;i++) {
-	    bcopy(Ximage->data,Ximage->data+i*Ximage->bytes_per_line,
-		Ximage->bytes_per_line);
-	}
+  int i;
+  if (fake_lut) {
+    int j;
+    for (j = 0; j < Ximage->height; j++)
+      for (i = 0; i < Ximage->width; i++)
+	XPutPixel(Ximage, i, j, cmap_pixtab[pixtab[0]]);
+  } else if (pixtab[0] != 0) {
+    /*
+     * Initialize image to pixtab[0].
+     */
+    if (colors == 2) {
+      for (i = 0; i < Ximage->bytes_per_line; i++) {
+	Ximage->data[i] = 0xff;
+      }
     } else {
-	/*
-	 * Initialize image to 0's.
-	 */
-	bzero(Ximage->data,Ximage->bytes_per_line*Ximage->height);
+      for (i = 0; i < Ximage->bytes_per_line; i++) {
+	Ximage->data[i] = pixtab[0];
+      }
     }
-    xlastcolor = -1;
-    XSetForeground(Xdp, Xgc, pixtab[0]);
-    if (onroot) {
-	XFillRectangle(Xdp,Xpixmap,Xgc,0,0,Xwinwidth,Xwinheight);
+    for (i = 1; i < Ximage->height; i++) {
+      bcopy(Ximage->data, Ximage->data+i*Ximage->bytes_per_line, 
+	    Ximage->bytes_per_line);
     }
-    XFillRectangle(Xdp,Xw,Xgc,0,0,Xwinwidth,Xwinheight);
-    xsync();
+  } else {
+    /*
+     * Initialize image to 0's.
+     */
+    bzero(Ximage->data, Ximage->bytes_per_line*Ximage->height);
+  }
+  xlastcolor = -1;
+  XSetForeground(Xdp, Xgc, FAKE_LUT(pixtab[0]));
+  if (onroot) {
+    XFillRectangle(Xdp, Xpixmap, Xgc, 0, 0, Xwinwidth, Xwinheight);
+  }
+  XFillRectangle(Xdp, Xw, Xgc, 0, 0, Xwinwidth, Xwinheight);
+  xsync();
+  drawing_or_drawn = 0;
 }
 
 /*
@@ -646,15 +717,35 @@ clearXwindow()
 static void
 initdacbox()
 {
-    int i;
-    for (i=0;i<256;i++) {
-	dacbox[i][0] = (i>>5)*8+7;
-	dacbox[i][1] = (((i+16)&28)>>2)*8+7;
-	dacbox[i][2] = (((i+2)&3))*16+15;
+    int i, j, k, s0, sp;
+
+    s0 = step & 1;
+    sp = step/2;
+
+    if (sp) {
+      --sp;
+      s0 = 1-s0;
+      for (i=0; i<256; i++) {
+         for (j=0; j<3; j++) {
+           k = (i*(cyclic[sp][j])) & 127;
+           if (k < 64) dacbox[i][j] = k; else dacbox[i][j] = (127 - k);
+         }
+      }
+    } else {
+      for (i=0;i<256;i++) {
+  	dacbox[i][0] = (i>>5)*8+7;
+  	dacbox[i][1] = (((i+16)&28)>>2)*8+7;
+  	dacbox[i][2] = (((i+2)&3))*16+15;
+      }
+      dacbox[0][0] = dacbox[0][1] = dacbox[0][2] = 0;
+      dacbox[1][0] = dacbox[1][1] = dacbox[1][2] = 63;
+      dacbox[2][0] = 47; dacbox[2][1] = dacbox[2][2] = 63;
     }
-    dacbox[0][0] = dacbox[0][1] = dacbox[0][2] = 0;
-    dacbox[1][0] = dacbox[1][1] = dacbox[1][2] = 63;
-    dacbox[2][0] = 47; dacbox[2][1] = dacbox[2][2] = 63;
+    if (s0)
+      for (i=0; i<256; i++)
+         for (j=0; j<3; j++)
+             dacbox[i][j] = 63 - dacbox[i][j];
+
 }
 
 
@@ -693,17 +784,26 @@ resizeWindow()
     unsigned int junkui;
     Window junkw;
     unsigned int width, height;
-    int Xmwidth;
-    Status status;
+    int Xmwidth, Xpad;
 
     if (unixDisk) return 0;
-
-    XGetGeometry(Xdp,Xw,&junkw,&junki, &junki, &width, &height,
+    if (resize_flag) {
+       Window root, parent, *children;
+       resize_flag = 0;
+       XQueryTree(Xdp, Xw, &root, &parent, &children, &junkui);
+       if (!parent) return 0;
+       XGetGeometry(Xdp, parent, &root, &junki, &junki,
+           &width, &height, &junkui, &junkui);
+       XResizeWindow(Xdp, Xw, width, height);
+       XSync(Xdp, False);
+       usleep(100000);
+    } else
+       XGetGeometry(Xdp,Xw,&junkw,&junki, &junki, &width, &height,
 	    &junkui, &junkui);
 
     if (oldx != width || oldy != height) {
-	sxdots = width;
-	sydots = height;
+	sxdots = width & -4;
+	sydots = height & -4;
 	videotable[0].xdots = sxdots;
 	videotable[0].ydots = sydots;
 	oldx = sxdots;
@@ -712,19 +812,28 @@ resizeWindow()
 	Xwinheight = sydots;
 	screenaspect = sydots/(float)sxdots;
 	finalaspectratio = screenaspect;
-	Xmwidth = (Xdepth > 1)? sxdots: (1 + sxdots/8);
+     Xpad = 8;  /* default, unless changed below */
+        if (Xdepth==1)
+           Xmwidth = 1 + sxdots/8;
+        else if (Xdepth<=8)
+           Xmwidth = sxdots;
+        else if (Xdepth<=16) {  /* 15 or 16 bpp */
+           Xmwidth = 2*sxdots;
+           Xpad = 16;
+        } else {  /* 24 or 32 bpp */
+           Xmwidth = 4*sxdots;
+           Xpad = 32;
+        }
 	if (pixbuf != NULL) {
 	    free(pixbuf);
 	}
 	pixbuf = (BYTE *) malloc(Xwinwidth *sizeof(BYTE));
-	if (Ximage != NULL) {
-	    free(Ximage->data);
+	if (Ximage != NULL)
 	    XDestroyImage(Ximage);
-	}
-	Ximage = XCreateImage(Xdp,Xvi,Xdepth, ZPixmap, 0, NULL, sxdots,
-	    sydots,8, Xmwidth);
+	Ximage = XCreateImage(Xdp, Xvi, Xdepth, ZPixmap, 0, NULL, sxdots,
+			      sydots, Xpad, Xmwidth);
 	if (Ximage == NULL) {
-	    printf("XCreateImage failed\n");
+	    fprintf(stderr,"XCreateImage failed\n");
 	    UnixDone();
 	    exit(-1);
 	}
@@ -759,66 +868,70 @@ resizeWindow()
 static int
 xcmapstuff()
 {
-    int ncells,i,powr;
+  int ncells, i, powr;
 
-    if (onroot) {
-	privatecolor = 0;
-    }
-    for (i=0;i<colors;i++) {
-	pixtab[i] = i;
-	ipixtab[i] = 999;
-    }
-    if (!gotrealdac) {
-    } else if (sharecolor) {
-	gotrealdac = 0;
-    } else if (privatecolor) {
-	Xcmap = XCreateColormap(Xdp,Xw,Xvi,AllocAll);
-	XSetWindowColormap(Xdp,Xw,Xcmap);
-    } else {
-	Xcmap = DefaultColormap(Xdp,Xdscreen);
-	for (powr=Xdepth;powr>=1;powr--) {
-	    ncells = 1<<powr;
-	    if (ncells>colors) continue;
-	    if (XAllocColorCells(Xdp,Xcmap,False,NULL,0,pixtab,
-		    (unsigned int)ncells)) {
-		colors = ncells;
-		printf("%d colors\n",colors);
-		usepixtab = 1;
-		break;
-	    }
-	}
-	if (!usepixtab) {
-	    printf("Couldn't allocate any colors\n");
-	    gotrealdac = 0;
-	}
-    }
-    for (i=0;i<colors;i++) {
-	ipixtab[pixtab[i]] = i;
-    }
-    /* We must make sure if any color uses position 0, that it is 0.
-     * This is so we can clear the image with bzero.
-     * So, suppose fractint 0 = cmap 42, cmap 0 = fractint 55.
-     * Then want fractint 0 = cmap 0, cmap 42 = fractint 55.
-     * I.e. pixtab[55] = 42, ipixtab[42] = 55.
-     */
-    if (ipixtab[0] == 999) {
-	ipixtab[0] = 0;
-    } else if (ipixtab[0] != 0) {
-	int other;
-	other = ipixtab[0];
-	pixtab[other] = pixtab[0];
-	ipixtab[pixtab[other]] = other;
-	pixtab[0] = 0;
-	ipixtab[0] = 0;
-    }
-
-    if (!gotrealdac && colors==2 && BlackPixelOfScreen(Xsc)!=0) {
-	pixtab[0] = ipixtab[0] = 1;
-	pixtab[1] = ipixtab[1] = 0;
+  if (onroot) {
+    privatecolor = 0;
+  }
+  for (i = 0; i < colors; i++) {
+    pixtab[i] = i;
+    ipixtab[i] = 999;
+  }
+  if (!gotrealdac) {
+    Xcmap = DefaultColormapOfScreen(Xsc);
+    if (fake_lut)
+      writevideopalette();
+  } else if (sharecolor) {
+    gotrealdac = 0;
+  } else if (privatecolor) {
+    Xcmap = XCreateColormap(Xdp, Xw, Xvi, AllocAll);
+    XSetWindowColormap(Xdp, Xw, Xcmap);
+  } else {
+    Xcmap = DefaultColormap(Xdp, Xdscreen);
+    for (powr = Xdepth; powr >= 1; powr--) {
+      ncells = 1 << powr;
+      if (ncells > colors)
+	continue;
+      if (XAllocColorCells(Xdp, Xcmap, False, NULL, 0, pixtab,
+			   (unsigned int) ncells)) {
+	colors = ncells;
+	fprintf(stderr,"%d colors\n", colors);
 	usepixtab = 1;
+	break;
+      }
     }
+    if (!usepixtab) {
+      fprintf(stderr,"Couldn't allocate any colors\n");
+      gotrealdac = 0;
+    }
+  }
+  for (i = 0; i < colors; i++) {
+    ipixtab[pixtab[i]] = i;
+  }
+  /* We must make sure if any color uses position 0, that it is 0.
+   * This is so we can clear the image with bzero.
+   * So, suppose fractint 0 = cmap 42, cmap 0 = fractint 55.
+   * Then want fractint 0 = cmap 0, cmap 42 = fractint 55.
+   * I.e. pixtab[55] = 42, ipixtab[42] = 55.
+   */
+  if (ipixtab[0] == 999) {
+    ipixtab[0] = 0;
+  } else if (ipixtab[0] != 0) {
+    int other;
+    other = ipixtab[0];
+    pixtab[other] = pixtab[0];
+    ipixtab[pixtab[other]] = other;
+    pixtab[0] = 0;
+    ipixtab[0] = 0;
+  }
 
-    return colors;
+  if (!gotrealdac && colors == 2 && BlackPixelOfScreen(Xsc) != 0) {
+    pixtab[0] = ipixtab[0] = 1;
+    pixtab[1] = ipixtab[1] = 0;
+    usepixtab = 1;
+  }
+
+  return colors;
 }
 /*
  *----------------------------------------------------------------------
@@ -844,6 +957,8 @@ BYTE *pixels;
     int i;
     BYTE *pixline;
 
+    drawing_or_drawn = 1;
+
 #if 1
     if (x==lastx) {
 	writevideo(x,y,pixels[0]);
@@ -858,8 +973,8 @@ BYTE *pixels;
     } else {
 	pixline = pixels;
     }
-    for (i=0;i<width;i++) {
-	XPutPixel(Ximage,x+i,y,pixline[i]);
+    for (i = 0; i < width; i++) {
+	XPutPixel(Ximage, x+i, y, FAKE_LUT(pixline[i]));
     }
     if (fastmode==1 && helpmode != HELPXHAIR) {
 	if (!alarmon) {
@@ -874,7 +989,7 @@ BYTE *pixels;
 #else
     width = lastx-x+1;
     for (i=0;i<width;i++) {
-	writevideo(x+i,y,pixels[i]);
+	writevideo(x+i,y,(int)pixels[i]);
     }
 #endif
 }
@@ -901,7 +1016,7 @@ BYTE *pixels;
     int i,width;
     width = lastx-x+1;
     for (i=0;i<width;i++) {
-	pixels[i] = readvideo(x+i,y);
+	pixels[i] = (BYTE)readvideo(x+i,y);
     }
 }
 
@@ -925,18 +1040,18 @@ int x,y,color;
 {
 #ifdef DEBUG /* Debugging checks */
     if (color>=colors || color < 0) {
-	printf("Color %d too big %d\n", color, colors);
+	fprintf(stderr,"Color %d too big %d\n", color, colors);
     }
     if (x>=sxdots || x<0 || y>=sydots || y<0) {
-	printf("Bad coord %d %d\n", x,y);
+	fprintf(stderr,"Bad coord %d %d\n", x,y);
     }
 #endif
     if (xlastcolor != color) {
-	XSetForeground(Xdp,Xgc,pixtab[color]);
+	XSetForeground(Xdp, Xgc, FAKE_LUT(pixtab[color]));
 	xlastcolor = color;
     }
-    XPutPixel(Ximage,x,y,pixtab[color]);
-    if (fastmode==1 && helpmode != HELPXHAIR) {
+    XPutPixel(Ximage, x, y, FAKE_LUT(pixtab[color]));
+    if (fastmode == 1 && helpmode != HELPXHAIR) {
 	if (!alarmon) {
 	    schedulealarm(0);
 	}
@@ -965,7 +1080,20 @@ int x,y,color;
  */
 int readvideo(int x, int y)
 {
-    return ipixtab[XGetPixel(Ximage,x,y)];
+#ifdef DEBUG /* Debugging checks */
+    if (x>=sxdots || x<0 || y>=sydots || y<0) {
+	fprintf(stderr,"Bad coord %d %d\n", x,y);
+    }
+#endif
+  if (fake_lut) {
+    int i;
+    XPixel pixel = XGetPixel(Ximage, x, y);
+    for (i = 0; i < colors; i++)
+      if (cmap_pixtab[i] == pixel)
+         return i;
+    return 0;
+  } else
+    return ipixtab[XGetPixel(Ximage, x, y)];
 }
 
 XColor cols[256];
@@ -989,8 +1117,8 @@ int readvideopalette()
 {
 
     int i;
-    if (gotrealdac==0) return -1;
-    for (i=0;i<256;i++) {
+    if (gotrealdac==0 && istruecolor && truemode) return -1;
+    for (i=0;i<colors;i++) {
 	dacbox[i][0] = cols[i].red/1024;
 	dacbox[i][1] = cols[i].green/1024;
 	dacbox[i][2] = cols[i].blue/1024;
@@ -1004,7 +1132,7 @@ int readvideopalette()
  *
  * writevideopalette --
  *	Writes dacbox into the video palette.
- *	
+ *
  *
  * Results:
  *	None.
@@ -1016,23 +1144,72 @@ int readvideopalette()
  */
 int writevideopalette()
 {
-    int i;
+  int i;
 
-    if (!gotrealdac) return -1;
-    for(i=0;i<256;i++) {
-	cols[i].pixel = pixtab[i];
-	cols[i].flags = DoRed | DoGreen | DoBlue;
-	cols[i].red = dacbox[i][0]*1024;
-	cols[i].green = dacbox[i][1]*1024;
-	cols[i].blue = dacbox[i][2]*1024;
+  if (!gotrealdac) {
+    if (fake_lut) {
+      /* !gotrealdac, fake_lut => truecolor, directcolor displays */
+      static unsigned char last_dac[256][3];
+      static int last_dac_inited = False;
+
+      for (i = 0; i < colors; i++) {
+	if (!last_dac_inited ||
+	    last_dac[i][0] != dacbox[i][0] ||
+	    last_dac[i][1] != dacbox[i][1] ||
+	    last_dac[i][2] != dacbox[i][2]) {
+	  cols[i].flags = DoRed | DoGreen | DoBlue;
+	  cols[i].red = dacbox[i][0]*1024;
+	  cols[i].green = dacbox[i][1]*1024;
+	  cols[i].blue = dacbox[i][2]*1024;
+
+          /* This seems not to work in truecolor modes, so commented out!
+	  if (cmap_pixtab_alloced) {
+	    XFreeColors(Xdp, Xcmap, cmap_pixtab + i, 1, None);
+	  } */
+	  if (XAllocColor(Xdp, Xcmap, &cols[i])) {
+	    cmap_pixtab[i] = cols[i].pixel;
+	  } else {
+	    assert(1);
+	    fprintf(stderr,"Allocating color %d failed.\n", i);
+	  }
+
+	  last_dac[i][0] = dacbox[i][0];
+	  last_dac[i][1] = dacbox[i][1];
+	  last_dac[i][2] = dacbox[i][2];
+	}
+      }
+      cmap_pixtab_alloced = True;
+      last_dac_inited = True;
+    } else {
+      /* !gotrealdac, !fake_lut => static color, static gray displays */
+      assert(1);
+    }
+  } else {
+    /* gotrealdac => grayscale or pseudocolor displays */
+    for (i = 0; i < colors; i++) {
+      cols[i].pixel = pixtab[i];
+      cols[i].flags = DoRed | DoGreen | DoBlue;
+      cols[i].red = dacbox[i][0]*1024;
+      cols[i].green = dacbox[i][1]*1024;
+      cols[i].blue = dacbox[i][2]*1024;
     }
     if (!unixDisk) {
-	XStoreColors(Xdp,Xcmap,cols,colors);
-	XFlush(Xdp);
-    }
-    return 0;
+      XStoreColors(Xdp, Xcmap, cols, colors);
+      XFlush(Xdp);
 
+/* None of these changed the colors without redrawing the fractal
+XSetWindowColormap(Xdp, Xw, Xcmap);
+XDrawRectangle(Xdp, Xw, Xgc, 0, 0, Xwinwidth, Xwinheight);
+XPutImage(Xdp,Xw,Xgc,Ximage,0,0,0,0,Xwinwidth,Xwinheight);
+XCopyArea(Xdp,Xw,Xw,Xgc,0,0,Xwinwidth,Xwinheight,0,0);
+XFlush(Xdp);
+*/
+    }
+  }
+
+  return 0;
 }
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1058,7 +1235,7 @@ int mode;
 	XSetFunction(Xdp, Xgc, GXcopy);
 	xlastfcn = GXcopy;
     } else {
-	XSetForeground(Xdp, Xgc, colors-1);
+	XSetForeground(Xdp, Xgc, WhitePixel(Xdp, Xdscreen));
 	xlastcolor = -1;
 	XSetFunction(Xdp, Xgc, GXxor);
 	xlastfcn = GXxor;
@@ -1165,13 +1342,14 @@ xgetkey(block)
 int block;
 {
     static int skipcount = 0;
-    int ch,ch2;
+    int ch;
     fd_set reads;
     int status;
     static struct timeval tout;
     tout.tv_sec = 0;
     tout.tv_usec = 500000;
     while (1) {
+#if 1
 	if (input_pending()) {
 	    ch = getachar();
 	    if (ch == ESC) {
@@ -1180,6 +1358,7 @@ int block;
 		return translatekey(ch);
 	    }
 	}
+#endif
 
 	/* Don't check X events every time, since that is expensive */
 	skipcount++;
@@ -1193,7 +1372,11 @@ int block;
 	    ch = xbufkey;
 	    xbufkey = 0;
 	    skipcount = 9999; /* If we got a key, check right away next time */
-	    return translatekey(ch);
+	    if (ch == ESC) {
+		return handleesc();
+	    } else {
+		return translatekey(ch);
+	    }
 	}
 	if (!block) break;
 	FD_ZERO(&reads);
@@ -1244,8 +1427,10 @@ int ch;
 		return PAGE_DOWN;
 	    case CTL('O'):
 		return CTL_HOME;
+#if 0
 	    case CTL('E'):
 		return CTL_END;
+#endif
 	    case 'H':
 		return LEFT_ARROW;
 	    case 'L':
@@ -1291,9 +1476,11 @@ int ch;
 		return F1;
 	    case '@':
 		return F2;
-	    case '#':
+#if 0
+            case '#':
 		return F3;
-	    case '$':
+#endif
+            case '$':
 		return F4;
 	    case '%':
 		return F5;
@@ -1514,9 +1701,9 @@ extern void Cursor_SetPos();
 int XZoomWaiting = 0;
 
 #define SENS 1
-#define ABS(x) ((x)>0?(x):-(x))
-#define MIN(x,y) ((x)<(y)?(x):(y))
-#define SIGN(x) ((x)>0?1:-1)
+#define ABS(x) ((x) > 0?(x):-(x))
+#define MIN(x, y) ((x) < (y)?(x):(y))
+#define SIGN(x) ((x) > 0?1:-1)
 
 /*
  *----------------------------------------------------------------------
@@ -1533,12 +1720,13 @@ int XZoomWaiting = 0;
  *
  *----------------------------------------------------------------------
  */
-static int
+static void
 xhandleevents()
 {
     XEvent xevent;
-    static ctl_mode = 0;
-    static shift_mode = 0;
+    int drawn;
+    static int ctl_mode = 0;
+    static int shift_mode = 0;
     int bandx0,bandy0,bandx1,bandy1;
     static int bnum=0;
     static int lastx,lasty;
@@ -1550,12 +1738,13 @@ xhandleevents()
 
     while (XPending(Xdp) && !xbufkey) {
 	XNextEvent(Xdp,&xevent);
+
 	switch (((XAnyEvent *)&xevent)->type) {
 	    case KeyRelease:
 		{
 		char buffer[1];
 		KeySym keysym;
-		(void) XLookupString(&xevent,buffer,1,&keysym,NULL);
+		(void) XLookupString(&xevent.xkey,buffer,1,&keysym,NULL);
 		switch (keysym) {
 		    case XK_Control_L:
 		    case XK_Control_R:
@@ -1573,8 +1762,7 @@ xhandleevents()
 		int charcount;
 		char buffer[1];
 		KeySym keysym;
-		int compose;
-		charcount = XLookupString(&xevent,buffer,1,&keysym,NULL);
+		charcount = XLookupString(&xevent.xkey,buffer,1,&keysym,NULL);
 		switch (keysym) {
 		    case XK_Control_L:
 		    case XK_Control_R:
@@ -1672,7 +1860,47 @@ xhandleevents()
 			 xbufkey = ctl_mode ? CTL_MINUS : '-';
 			 return;
 			 break;
-		    case XK_Return:
+#if 0
+/* The following need to be somewhere else, otherwise these keys are not available */
+/* in any other mode.  For example, the '0' and '=' keys won't work with the <g> command */
+/* or the <b> command. JCO 06-23-2001 */
+                    case XK_0:
+		    case XK_KP_0:
+			 step = 0;
+			 initdacbox();
+			 if (drawing_or_drawn) xbufkey = 'D';
+			 return;
+		    case XK_exclam:
+			 step = (step & 126) + 1 - (step & 1);
+			 initdacbox();
+			 if (drawing_or_drawn) xbufkey = 'D';
+			 return;
+		    case XK_greater:
+			 step = (step+2) % 50;
+			 initdacbox();
+			 xbufkey = 'D';
+			 return;
+		    case XK_less:
+		         step = (step+48) % 50;
+			 initdacbox();
+			 if (drawing_or_drawn) xbufkey = 'D';
+			 return;
+		    case XK_parenright:
+			 step = (step+12) % 50;
+			 initdacbox();
+			 if (drawing_or_drawn) xbufkey = 'D';
+			 return;
+		    case XK_parenleft:
+			 step = (step+38) % 50;
+			 initdacbox();
+			 if (drawing_or_drawn) xbufkey = 'D';
+			 return;
+		    case XK_equal:
+		    case XK_KP_Equal:
+                         xbufkey = 'D';
+			 return;
+#endif
+                    case XK_Return:
 		    case XK_KP_Enter:
 			 xbufkey = ctl_mode ? CTL('T') : '\n';
 			 return;
@@ -1802,7 +2030,22 @@ xhandleevents()
 		drawbox(0);
 		}
 		break;
-	    case Expose:
+            case ConfigureNotify:
+	        XSelectInput(Xdp,Xw,KeyPressMask|KeyReleaseMask|ExposureMask|
+		    ButtonPressMask|ButtonReleaseMask|PointerMotionMask);
+	        resize_flag = 1;
+		drawn = drawing_or_drawn;
+		if (resizeWindow()) {
+		  if (drawn) {
+		     xbufkey = 'D';
+		     return;
+		  }
+		}
+	        XSelectInput(Xdp,Xw,KeyPressMask|KeyReleaseMask|ExposureMask|
+		    ButtonPressMask|ButtonReleaseMask|
+                    PointerMotionMask|StructureNotifyMask);
+		break;
+            case Expose:
 		if (!doesBacking) {
 		int x,y,w,h;
 		x = xevent.xexpose.x;
@@ -1824,8 +2067,13 @@ xhandleevents()
 		}
 		}
 		break;
-	}
-    }
+	    default:
+	        /* All events selected by StructureNotifyMask
+		 * except ConfigureNotify are thrown away here,
+		 * since nothing is done with them */
+	        break;
+	}  /* End switch */
+    }  /* End while */
 
     if (!xbufkey && editpal_cursor && !inside_help && lookatmouse == 3 &&
 	    (dx != 0 || dy != 0)) {
@@ -1996,57 +2244,74 @@ static unsigned char *fontPtr = NULL;
 unsigned char *
 xgetfont()
 {
-    XFontStruct *font_info;
-    Pixmap font_pixmap;
-    XImage *font_image;
-    char str[8];
-    int i,j,k,l;
-    int width;
-    fontPtr = (unsigned char *)malloc(128*8);
-    bzero(fontPtr,128*8);
-    XSetBackground(Xdp,Xgc,0);
-    xlastcolor = -1;
-    XSetForeground(Xdp,Xgc,1);
-#define FONT "-*-*-medium-r-*-*-9-*-*-*-*-*-iso8859-*"
-    font_info = XLoadQueryFont(Xdp,FONT);
-    if (font_info==NULL) {
-	printf("No %s\n", FONT);
-    }
-    if (font_info==NULL || font_info->max_bounds.width>8 ||
-	    font_info->max_bounds.width != font_info->min_bounds.width) {
-	printf("Bad font: %s\n", FONT);
-	sleep(2);
-	font_info = XLoadQueryFont(Xdp,"6x12");
-    }
-    if (font_info==NULL) return NULL;
-    width = font_info->max_bounds.width;
-    if (font_info->max_bounds.width>8 ||
-	    font_info->max_bounds.width != font_info->min_bounds.width) {
-	printf("Bad font\n");
-	return NULL;
-    }
-    font_pixmap = XCreatePixmap(Xdp,Xw,64,8,Xdepth);
+  XFontStruct *font_info;
+  XImage *font_image;
+  char str[8];
+  int i, j, k, l;
+  int width;
+  Pixmap font_pixmap;
+  XGCValues values;
+  GC font_gc;
 
-    for (i=0;i<128;i+=8) {
-	for (j=0;j<8;j++) {
-	    str[j] = i+j;
-	}
-	XDrawImageString(Xdp,font_pixmap,Xgc,0,8,str,8);
-	font_image = XGetImage(Xdp,font_pixmap,0,0,64,8,AllPlanes,XYPixmap);
-	for (j=0;j<8;j++) {
-	    for (k=0;k<8;k++) {
-		for (l=0;l<width;l++) {
-		    if (XGetPixel(font_image,j*width+l,k)) {
-			fontPtr[(i+j)*8+k] = (fontPtr[(i+j)*8+k]<<1)|1;
-		    } else {
-			fontPtr[(i+j)*8+k] = (fontPtr[(i+j)*8+k]<<1);
-		    }
-		}
-	    }
-	}
-	XDestroyImage(font_image);
+  fontPtr = (unsigned char *)malloc(128*8);
+  bzero(fontPtr, 128*8);
+
+  xlastcolor = -1;
+#define FONT "-*-*-medium-r-*-*-9-*-*-*-*-*-iso8859-*"
+  font_info = XLoadQueryFont(Xdp, FONT);
+  if (font_info == NULL) {
+    printf("No %s\n", FONT);
+  }
+  if (font_info == NULL || font_info->max_bounds.width > 8 ||
+      font_info->max_bounds.width != font_info->min_bounds.width) {
+    printf("Bad font: %s\n", FONT);
+    sleep(2);
+    font_info = XLoadQueryFont(Xdp, "6x12");
+  }
+  if (font_info == NULL) return NULL;
+  width = font_info->max_bounds.width;
+  if (font_info->max_bounds.width > 8 ||
+      font_info->max_bounds.width != font_info->min_bounds.width) {
+    printf("Bad font\n");
+    return NULL;
+  }
+
+  font_pixmap = XCreatePixmap(Xdp, Xw, 64, 8, Xdepth);
+  assert(font_pixmap);
+  values.background = 0;
+  values.foreground = 1;
+  values.font = font_info->fid;
+  font_gc = XCreateGC(Xdp, font_pixmap,
+		      GCForeground | GCBackground | GCFont, &values);
+  assert(font_gc);
+
+  for (i = 0; i < 128; i+=8) {
+    for (j = 0; j < 8; j++) {
+      str[j] = i+j;
     }
-    return fontPtr;
+
+    XDrawImageString(Xdp, font_pixmap, Xgc, 0, 8, str, 8);
+
+    font_image = XGetImage(Xdp, font_pixmap, 0, 0, 64, 8, AllPlanes, XYPixmap);
+    assert(font_image);
+    for (j = 0; j < 8; j++) {
+      for (k = 0; k < 8; k++) {
+	for (l = 0; l < width; l++) {
+	  if (XGetPixel(font_image, j*width+l, k)) {
+	    fontPtr[(i+j)*8+k] = (fontPtr[(i+j)*8+k] << 1) | 1;
+	  } else {
+	    fontPtr[(i+j)*8+k] = (fontPtr[(i+j)*8+k] << 1);
+	  }
+	}
+      }
+    }
+    XDestroyImage(font_image);
+  }
+
+  XFreeGC(Xdp, font_gc);
+  XFreePixmap(Xdp, font_pixmap);
+
+  return fontPtr;
 }
 
 /*
@@ -2068,12 +2333,12 @@ xgetfont()
 void
 shell_to_dos()
 {
-    __sighandler_t sigint;
+    SignalHandler sigint;
     char *shell;
     char *argv[2];
     int pid, donepid;
 
-    sigint = (__sighandler_t)signal(SIGINT, SIG_IGN);
+    sigint = (SignalHandler)signal(SIGINT, SIG_IGN);
     shell = getenv("SHELL");
     if (shell==NULL) {
 	shell = SHELL;
@@ -2121,7 +2386,7 @@ shell_to_dos()
 	fcntl(0,F_SETFL,FNDELAY);
     }
 
-    signal(SIGINT, (__sighandler_t)sigint);
+    signal(SIGINT, (SignalHandler)sigint);
     putchar('\n');
 }
 
@@ -2146,7 +2411,7 @@ schedulealarm(soon)
 int soon;
 {
     if (!fastmode) return;
-    signal(SIGALRM, (__sighandler_t)setredrawscreen);
+    signal(SIGALRM, (SignalHandler)setredrawscreen);
     if (soon) {
 	alarm(1);
     } else {
@@ -2194,13 +2459,17 @@ setredrawscreen()
 void
 redrawscreen()
 {
-    if (alarmon) {
-	XPutImage(Xdp,Xw,Xgc,Ximage,0,0,0,0,sxdots,sydots);
-	if (onroot) {
-	    XPutImage(Xdp,Xpixmap,Xgc,Ximage,0,0,0,0,sxdots,sydots);
-	}
-	alarmon = 0;
+  if (alarmon) {
+    XPutImage(Xdp, Xw, Xgc, Ximage, 0, 0, 0, 0, sxdots, sydots);
+    if (onroot) {
+      XPutImage(Xdp, Xpixmap, Xgc, Ximage, 0, 0, 0, 0, sxdots, sydots);
     }
-    doredraw = 0;
+    alarmon = 0;
+  }
+  doredraw = 0;
+  if (!unixDisk)
+    XSelectInput(Xdp,Xw,KeyPressMask|KeyReleaseMask|ExposureMask|
+	    ButtonPressMask|ButtonReleaseMask|
+            PointerMotionMask|StructureNotifyMask);
 }
 
