@@ -41,26 +41,30 @@ int boxcolor;
 void dispbox(void)
 {
    int i;
+   int boxc = (colors-1)&boxcolor;
+   unsigned char *values = (unsigned char *)boxvalues;
+   int rgb[3];
    xorTARGA = 1;
    for(i=0;i<boxcount;i++)
    {
-      if(istruecolor)
+      if(istruecolor && truemode)
       {
-         int rgb[3];
-         gettruecolor(boxx[i]-sxoffs,
-                      boxy[i]-syoffs,&rgb[0],&rgb[1],&rgb[2]);
-         puttruecolor(boxx[i]-sxoffs,
-                      boxy[i]-syoffs,rgb[0]^255,rgb[1]^255,rgb[2]^255);
+         gettruecolor(boxx[i]-sxoffs,boxy[i]-syoffs,&rgb[0],&rgb[1],&rgb[2]);
+         puttruecolor(boxx[i]-sxoffs,boxy[i]-syoffs,
+                      rgb[0]^255,rgb[1]^255,rgb[2]^255);
       }
       else
-      {
-         unsigned char *values = (unsigned char *)boxvalues;
-         int boxc = (colors-1)&boxcolor;
-         values[i] = (unsigned char)
-         getcolor(boxx[i]-sxoffs,boxy[i]-syoffs);
-         putcolor(boxx[i]-sxoffs,boxy[i]-syoffs,boxc);
-      }
+         values[i] = (unsigned char)getcolor(boxx[i]-sxoffs,boxy[i]-syoffs);
    }
+/* There is an interaction between getcolor and putcolor, so separate them */
+   if (!(istruecolor && truemode)) /* don't need this for truecolor with truemode set */
+      for(i=0;i<boxcount;i++)
+      {
+         if (colors == 2)
+            putcolor(boxx[i]-sxoffs,boxy[i]-syoffs,(1 - values[i]));
+         else
+            putcolor(boxx[i]-sxoffs,boxy[i]-syoffs,boxc);
+      }
    xorTARGA = 0;
 }
 
@@ -68,7 +72,7 @@ void clearbox(void)
 {
    int i;
    xorTARGA = 1;
-   if(istruecolor)
+   if(istruecolor && truemode)
    {
       dispbox();
    }
@@ -90,7 +94,7 @@ void drawbox(int drawit)
     double tmpx,tmpy,dx,dy,rotcos,rotsin,ftemp1,ftemp2;
     double fxwidth,fxskew,fydepth,fyskew,fxadj;
     bf_t bffxwidth, bffxskew, bffydepth, bffyskew, bffxadj;
-    int saved;
+    int saved=0;
     if (zwidth==0) { /* no box to draw */
         if (boxcount!=0) { /* remove the old box from display */
             clearbox(); 
@@ -301,6 +305,25 @@ void moveboxf(double dx, double dy)
             row -= row & (align-1);
             zby = (double)row/dysize; }
         }
+#ifndef XFRACT
+    if (video_scroll != 0) {  /* scroll screen center to the box center */
+        col = (int)((zbx + zwidth/2)*(dxsize + PIXELROUND)) + sxoffs;
+        row = (int)((zby + zdepth/2)*(dysize + PIXELROUND)) + syoffs;
+        switch (zscroll) {
+            case 0:  /* fixed - screen center fixed to the zoombox center */
+                scroll_center(col,row);
+                break;
+            case 1:  /* relaxed - as the zoombox center leaves the screen */
+                if ((col -= video_startx) > 0 && (col -= vesa_xres - 1) < 0)
+                    col = 0;
+                if ((row -= video_starty) > 0 && (row -= vesa_yres - 1) < 0)
+                    row = 0;
+                if (col != 0 || row != 0)
+                    scroll_relative(col, row);
+                break;
+            }
+        }
+#endif
     }
 
 static void _fastcall chgboxf(double dwidth, double ddepth)
@@ -560,7 +583,7 @@ static int check_pan(void) /* return 0 if can't, alignment requirement if can */
         }
     /* solid guessing */
     start_resume();
-    get_resume(sizeof(int),&num_worklist,sizeof(worklist),worklist,0);
+    get_resume(sizeof(num_worklist),&num_worklist,sizeof(worklist),worklist,0);
     /* don't do end_resume! we're just looking */
     i = 9;
     for (j=0; j<num_worklist; ++j) /* find lowest pass in any pending window */
@@ -612,7 +635,7 @@ int init_pan_or_recalc(int do_zoomout) /* decide to recalc, or to chg worklist &
     num_worklist = 0;
     if (calc_status == 2) {
        start_resume();
-       get_resume(sizeof(int),&num_worklist,sizeof(worklist),worklist,0);
+       get_resume(sizeof(num_worklist),&num_worklist,sizeof(worklist),worklist,0);
        } /* don't do end_resume! we might still change our mind */
     /* adjust existing worklist entries */
     for (i=0; i<num_worklist; ++i) {
@@ -654,8 +677,8 @@ Cancel resumes old image, continue pans and calculates a new one."};
     else         /* move image down */
         for (y=ydots; --y>=0;)  move_row(y+row,y,col);
     fix_worklist(); /* fixup any out of bounds worklist entries */
-    alloc_resume(sizeof(worklist)+10,1); /* post the new worklist */
-    put_resume(sizeof(int),&num_worklist,sizeof(worklist),worklist,0);
+    alloc_resume(sizeof(worklist)+20,2); /* post the new worklist */
+    put_resume(sizeof(num_worklist),&num_worklist,sizeof(worklist),worklist,0);
     return(0);
     }
 
@@ -686,9 +709,10 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
             --num_worklist;
             --i;
             continue; }
-        if (wk->yystart < 0) /* partly off top edge */
-            if ((wk->sym&1) == 0) /* no sym, easy */
+        if (wk->yystart < 0) { /* partly off top edge */
+            if ((wk->sym&1) == 0) { /* no sym, easy */
                 wk->yystart = 0;
+                wk->xxbegin = 0; }
             else { /* xaxis symmetry */
                 if ((j = wk->yystop + wk->yystart) > 0
                   && num_worklist < MAXCALCWORK) { /* split the sym part */
@@ -699,11 +723,12 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
                 else
                     wk->yystart = 0;
                 restart_window(i); /* restart the no-longer sym part */
-                }
+            }
+        }
         if (wk->yystop >= ydots) { /* partly off bottom edge */
            j = ydots-1;
            if ((wk->sym&1) != 0) { /* uses xaxis symmetry */
-              if ((k = wk->yystart + (wk->yystop - j)) < j)
+              if ((k = wk->yystart + (wk->yystop - j)) < j) {
                  if (num_worklist >= MAXCALCWORK) /* no room to split */
                     restart_window(i);
                  else { /* split it */
@@ -711,9 +736,10 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
                     worklist[num_worklist].yystart = k;
                     worklist[num_worklist++].yystop = j;
                     j = k-1; }
+              }
               wk->sym &= -1 - 1; }
            wk->yystop = j; }
-        if (wk->xxstart < 0) /* partly off left edge */
+        if (wk->xxstart < 0) { /* partly off left edge */
             if ((wk->sym&2) == 0) /* no sym, easy */
                 wk->xxstart = 0;
             else { /* yaxis symmetry */
@@ -726,11 +752,12 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
                 else
                     wk->xxstart = 0;
                 restart_window(i); /* restart the no-longer sym part */
-                }
+            }
+        }
         if (wk->xxstop >= xdots) { /* partly off right edge */
            j = xdots-1;
            if ((wk->sym&2) != 0) { /* uses xaxis symmetry */
-              if ((k = wk->xxstart + (wk->xxstop - j)) < j)
+              if ((k = wk->xxstart + (wk->xxstop - j)) < j) {
                  if (num_worklist >= MAXCALCWORK) /* no room to split */
                     restart_window(i);
                  else { /* split it */
@@ -738,6 +765,7 @@ static void fix_worklist(void) /* fix out of bounds and symmetry related stuff *
                     worklist[num_worklist].xxstart = k;
                     worklist[num_worklist++].xxstop = j;
                     j = k-1; }
+              }
               wk->sym &= -1 - 2; }
            wk->xxstop = j; }
         if (wk->yybegin < wk->yystart) wk->yybegin = wk->yystart;
